void Fit(TH1F* hmass, string Energy);

//////////////////////////////////////////////////////
void GetYieldEvolution(){

  //TFile* ifile = new TFile("histo_mass_all.root","read");
  TFile* ifile = new TFile("histo_mass_part2.root","read");

  TH2F* hnorm = new TH2F("hnorm","hnorm",20,0,20,1000,80,160);
  TH2F* hyield = new TH2F("hyield","hyield",20,0,20,1000,80,160);

  TString histo_name = "hmass_vs_ex";
  TH2F* h2 = (TH2F*)ifile->FindObjectAny(histo_name);
  
  h2->Draw("colz");

  double nentries[20];
  for(int i=0; i<h2->GetNbinsX(); i++){
    h2->ProjectionY("h2_py",i,i);
    TH1F* hpy = (TH1F*)gDirectory->FindObjectAny("h2_py");
    
    nentries[i] = hpy->Integral(1,h2->GetNbinsY());

    cout << i << " " << nentries[i] << endl;
  }

  // Getting norm
  for(int i=0; i<hnorm->GetNbinsX(); i++){
    for(int j=0; j<hnorm->GetNbinsY(); j++){
      hnorm->SetBinContent(i,j,nentries[i]);
    }
  }

  // Filling normalize histogram
  for(int i=6; i<hnorm->GetNbinsX(); i++){
    for(int j=0; j<hnorm->GetNbinsY(); j++){
      double val_orig = h2->GetBinContent(i,j);
      double val_error = sqrt(val_orig);
      double val_norm = hnorm->GetBinContent(i,j); 
      double new_val = val_orig/val_norm*200;
      hyield->SetBinContent(i,j,new_val);
      hyield->SetBinError(i,j,val_error/val_norm*200);
    }
  }

  hyield->Draw("colz");
    
}

//////////////////////////////////////////////////////
void Fit(TH1F* hmass,string Energy){
  int rebin = 2;
  hmass->Rebin(rebin);
  hmass->Draw();

  int Amin = 82;
  int Amax = 153;

  int A = Amin;

  int NumberOfA = Amax - Amin;
  Double_t para[3*NumberOfA];

  TF1* g[NumberOfA];
  double Integral[NumberOfA];
  double Integral_err[NumberOfA];
  double total_integral = 0;
  double Yield[NumberOfA];

  for(int i=0; i<NumberOfA; i++){
    g[i] = new TF1(Form("g%i",i),"gaus",A-0.4,A+0.4);   
    if(i==0){
      g[i]->SetParameter(0,100);
      g[i]->SetParameter(1,Amin);
      g[i]->SetParameter(2,0.3);
    }
    else if(i>0){
      g[i]->SetParameter(0,g[i-1]->GetParameter(0));
      g[i]->SetParameter(1,g[i-1]->GetParameter(1));
      g[i]->SetParameter(2,g[i-1]->GetParameter(2));
    
      g[i]->SetParLimits(1,A-0.15,A+0.15);
      g[i]->SetParLimits(2,0.1,0.35);

    }
    
    hmass->Fit(g[i],"qR");

    g[i]->Draw("lsame");
    g[i]->GetParameters(&para[3*i]);

    A++;
  }


  TString total_func = "gaus(0)";
  for(int i=1; i<NumberOfA; i++){
    TString gaus_func = Form("gaus(%i)",3*i);
    total_func += "+" + gaus_func;
  }

  TF1* total = new TF1("total",total_func,Amin-0.5, Amin+NumberOfA-0.5);

  A = Amin;
  for(int i=0; i<NumberOfA; i++){
    total->SetParameter(3*i,g[i]->GetParameter(0));
    total->SetParameter(3*i+1,g[i]->GetParameter(1));
    total->SetParameter(3*i+2,g[i]->GetParameter(2));
    total->SetParLimits(3*i+1,A-0.15,A+0.15);
    total->SetParLimits(3*i+2,0.1,0.35);
    //cout << A << " " << g[i]->GetParameter(2) << endl;
    A++;
  }
  total->SetParameters(para);
  total->SetLineColor(4);

  hmass->Fit(total,"Rq");

  A = Amin;
  TF1* one_gauss = new TF1("one_gaus","gaus");
  for(int i=0; i<NumberOfA; i++){
      
    double Amplitude = total->GetParameter(3*i);
    double mean = total->GetParameter(3*i+1);
    double sigma = total->GetParameter(3*i+2);
    double Amplitude_err = total->GetParError(3*i);
    double mean_err = total->GetParError(3*i+1);
    double sigma_err = total->GetParError(3*i+2);
    
    one_gauss->SetParameter(0,Amplitude);
    one_gauss->SetParameter(1,mean);
    one_gauss->SetParameter(2,sigma);
    one_gauss->SetParError(0,Amplitude_err);
    one_gauss->SetParError(1,mean_err);
    one_gauss->SetParError(2,sigma_err);

    //cout << "A=  " << A << endl;
    //cout << "Amplitude= " << Amplitude << "+/-" << Amplitude_err << endl;
    //cout << "Mean= " << mean  << endl;
    //cout << "Sigma= " << sigma << "+/-" << sigma_err << endl;

    

    Integral[i] = (12.5/rebin)*Amplitude*sigma*sqrt(2*TMath::Pi());
    Integral_err[i] = (12.5/rebin)*sqrt(2*TMath::Pi()*(pow(sigma*Amplitude_err,2) + pow(Amplitude*sigma_err,2)));
    //Integral_err[i] = sqrt(Integral[i]);
    
    cout << "**************** Compare integral " << endl;
    cout << Integral[i] << " / " << (12.5/rebin)*one_gauss->Integral(A-1,A+1) << endl;
    cout << Integral_err[i] << " / " << one_gauss->IntegralError(A-1,A+1) << endl;

    total_integral += Integral[i];
    A++;
  }

  double histo_integral = hmass->Integral();
  hmass->Write();
  total->Write();

  string output_filename = "Mass_Yield_" + Energy + ".dat";
  ofstream ofile;
  ofile.open(output_filename.c_str());

  // Yield calculation //
  A = Amin;
  for(int i=0; i<NumberOfA; i++){
    Yield[i] = Integral[i]/total_integral*200;
    //Yield[i] = Integral[i]/histo_integral*200;
    double yield_err = Integral_err[i]/total_integral*200;
    //double yield_err = Integral_err[i]/histo_integral*200;
    ofile << A << " " << Yield[i] << " " << yield_err << endl;
    if(A==99) cout << "A=99 -> Y= " << Yield[i] << endl;
    if(A==147) cout << "A=147 -> Y= " << Yield[i] << endl;
    A++;
  }
  ofile.close();


}
