#include <vector>
double TransfoX( double X, vector<double> &MinX ,vector<double> &MaxX ,vector<double> &ScalingBin ,vector<double> &NewMinX ,vector<double> &NewMaxX , vector<double> &Scale, int BinPerMM);

void GetBin(vector<double>&NBin, vector<TH1F*>&OldPosX , int StartBin , int BinPerMM ,int DetNbr );

void CalculateNewBin(vector<double> &MinX,vector<double> &MaxX,vector<double> &NewMinX, vector<double> &NewMaxX ,vector<double> &BinWidth, vector<double> &ScalingBin, vector<double> &Scale, vector<double>&NBin, int BinPerMM , int mm );


void Linearisation3(int RunStart, int RunStop){
    //========================================================================
    //-------------------Open tree and define Hist----------------------------
    //======================================================================== 

    TChain *ch = new TChain("PhysicsTree"); 


    for(int run = RunStart ; run <= RunStop ; run++){
        ch->Add(Form("../../root/analysis/linea_%d.root",run));
    } //End of run loop

    std::vector<TH1F*> PositionX,PositionY,OldPosX(4),OldPosY(4);
    for (int i=1; i<=4; i++ ){
        std::string PosXName = "hPositionXDet" + std::to_string(i) ;
        std::string PosXTitle = "PositionXDet" + std::to_string(i) ;
        std::string PosYName = "hPositionYDet" + std::to_string(i) ;
        std::string PosYTitle = "PositionYDet" + std::to_string(i) ;
        TH1F * PosX;
        TH1F * PosY;
        if ( i==1 ){
            PosX = new TH1F(PosXName.c_str(),PosXTitle.c_str(),1000,0,40);
            PosY = new TH1F(PosYName.c_str(),PosYTitle.c_str(),1000,0,60);
        }//End if det 1

        if ( i==2 ){
            PosX = new TH1F(PosXName.c_str(),PosXTitle.c_str(),1000,0,63);
            PosY = new TH1F(PosYName.c_str(),PosYTitle.c_str(),1000,0,92);

        }//End if det 2

        if ( i==3 || i==4 ) {
            PosX = new TH1F(PosXName.c_str(),PosXTitle.c_str(),1000,-1000,0);
            PosY = new TH1F(PosYName.c_str(),PosYTitle.c_str(),1000,0,160);
        }//End if det 3 or 4

        PositionX.push_back(PosX);
        PositionY.push_back(PosY);
    }//End of i loop
    TH1F *Det1 = new TH1F("Det1","Det1",1000,0,40);
    TH1F *Det1Adjusted = new TH1F("Det1Adjusted","Det1Adjusted",1000,0,40);
    TH1F *VarBin;
    




    //========================================================================
    //-----------------------Importing conditional var------------------------
    //========================================================================
    TFPMWPhysics *FPMW = new TFPMWPhysics();
    ch->SetBranchAddress("FPMW",&FPMW);


    TFile *InputHist = new TFile("HistFPMW.root","READ");
    if (!InputHist || InputHist->IsZombie()){
        cerr << "Error opening file !" << endl;
        return ;
    }
    for (int i=1; i<=4; i++ ){
        std::string PosXName = "hPositionXDet" + std::to_string(i) ;
        std::string PosXTitle = "PositionXDet" + std::to_string(i) ;
        std::string PosYName = "hPositionYDet" + std::to_string(i) ;
        std::string PosYTitle = "PositionYDet" + std::to_string(i) ;
        
        InputHist->GetObject(PosXName.c_str(),OldPosX[i-1]);
        InputHist->GetObject(PosYName.c_str(),OldPosY[i-1]);
    }//End of i loop


    //========================================================================
    //--------------------------TreatEvent------------------------------------
    //========================================================================
    int Nentries = ch->GetEntries();
    int GoodEntries = 0, Bin2= 0,Bin2Treated=0;
    bool Verbose = true;

    vector<vector<double>> MinXAll, MaxXAll, NewMinXAll, NewMaxXAll, NewBinWidthAll, ScalingBinAll, ScaleAll, NBinAll;
    vector<double> VarBinHolder;
    int NumberBins= OldPosX[0]->GetNbinsX();
    double WidthHist = OldPosX[0]->GetBinLowEdge(NumberBins) + OldPosX[0]->GetBinWidth(NumberBins) - OldPosX[0]->GetBinLowEdge(1);
    int CurrentBin=1, BinPerMM = NumberBins/WidthHist;
    
    //WidthHist=4;
    for (int mm = 0 ; mm <= WidthHist -1 ; mm ++ ){
        // Here we calculate the new bin edges
        size_t N = BinPerMM;
        vector<double> MinX, MaxX, NewMinX(N+1), NewMaxX(N), NewBinWidth, ScalingBin, Scale, NBin;
        int DetNbr = 0;
        GetBin(NBin,OldPosX,CurrentBin,BinPerMM,0);    

        CalculateNewBin(MinX, MaxX, NewMinX, NewMaxX , NewBinWidth , ScalingBin, Scale, NBin, BinPerMM , mm);

        MinXAll.push_back(MinX);
        MaxXAll.push_back(MaxX);
        if (OldPosX[0] -> GetBinContent(CurrentBin) == 0) {
            NewMinXAll.push_back(NewMinX);
            NewMaxXAll.push_back(NewMaxX);
            NewBinWidthAll.push_back(NewBinWidth);
            ScalingBinAll.push_back(ScalingBin);
            ScaleAll.push_back(Scale);
            NBinAll.push_back(NBin);      
        
        }

        if ( OldPosX[0] ->GetBinContent(CurrentBin)!=0){
            NewMinXAll.push_back(NewMinX);
            NewMaxXAll.push_back(NewMaxX);
            NewBinWidthAll.push_back(NewBinWidth);
            ScalingBinAll.push_back(ScalingBin);
            ScaleAll.push_back(Scale);
            NBinAll.push_back(NBin);

            for (int i=0; i<N ; i++){
                VarBinHolder.push_back(NewMinX[i]);
                if (Verbose == false && (mm==3|| mm==4) ){
                    cout << "MinX[i]   " << MinX[i];
                    cout << "  NewMinX[i]  " << NewMinX[i] << endl;
                    cout << " NBIN " << NBin[i] << endl;
                }
            }
        }
        else {
            for (int i=0 ; i<N ; i++){
                VarBinHolder.push_back(MinX[i]);
            }
        }
        CurrentBin+=BinPerMM;
    } //Loop on mm of the FPMW
    VarBinHolder.push_back(MaxXAll[MaxXAll.size()-1][24]);

    if( Verbose == false) {
        for (int i = 0 ; i< VarBinHolder.size(); i++) {
            cout << VarBinHolder[i] << endl;
        }
    }
    VarBin = new TH1F("VarBin", "VarBin", NumberBins, &VarBinHolder[0]);
    
    for (int entries=0; entries<Nentries; entries++){
        ch -> GetEntry(entries); 
        if(entries%100000==0){
            cout << "\033[92m\r Processing tree..." << (double)entries/Nentries*100 << "\% done" << flush;
        }//End cout loading


        if(FPMW->DetectorNbr.size()>0){
            GoodEntries += 1;
            for (int DetectorNbr = 0 ; DetectorNbr < FPMW->DetectorNbr.size() ; DetectorNbr ++){
                double X = FPMW->PositionX[DetectorNbr];
                double Y = FPMW->PositionY[DetectorNbr];

                if (DetectorNbr>=0 && DetectorNbr<4){
                    PositionX[DetectorNbr]->Fill(X); 
                    PositionY[DetectorNbr]->Fill(Y); 
                }


                if (DetectorNbr == 0) {
                    for (int mm =0 ; mm <= WidthHist -1 ; mm++){
                            if (X >= MinXAll[mm][0] && X< MaxXAll[mm][MaxXAll[mm].size()-1]){
                                double XNew = TransfoX(X, MinXAll[mm], MaxXAll[mm], ScalingBinAll[mm], NewMinXAll[mm], NewMaxXAll[mm], ScaleAll[mm], MinXAll[mm].size()-1);
                                VarBin->Fill(XNew);
                                Det1 -> Fill(X);
                                Det1Adjusted->Fill(XNew);
                                if (Verbose == false){
                                    cout << X << " " << XNew << "  " << MaxXAll[mm][MaxXAll[mm].size()-1]<< endl;
                                    cout << "************************"<<endl;   
                                }
                            }
                    }//end loop mm
                }// End detNbr == 0
            } // Loop detnbr
        }//Non empty event

    }//End loop on entries



    cout << endl << "Good entries" << GoodEntries << endl;

    TFile *OutputHist = new TFile("HistFPMW.root","RECREATE");
    std::vector<TCanvas*> canvas;
    for (int i=0; i<4; i++ ){
        std::string CanvasName = "c" + std::to_string(i) ;
        TCanvas *c = new TCanvas(CanvasName.c_str(),CanvasName.c_str(),960,540);
        c -> Divide(2,1);
        canvas.push_back(c);


        canvas[i] -> cd(1);
        PositionX[i] -> Draw("logY");
        PositionX[i] -> Write();
        gPad->SetLogy();
        canvas[i] -> cd(2);
        PositionY[i] -> Draw("logY");
        PositionY[i] -> Write();
        gPad->SetLogy();

    }
    TCanvas * CZoom = new TCanvas("CZoom","CZoom",1860,540);
    CZoom -> Divide(3,1);
    CZoom -> cd(1);
    Det1 -> Draw();
    CZoom -> cd(2);
    Det1Adjusted -> Draw("");
    Det1Adjusted -> Write();
    CZoom -> cd(3);
    VarBin -> Draw("");
    VarBin -> Write();
    gStyle-> SetOptStat(11111111);


    OutputHist->Close();
    InputHist-> Close();



} // end macro

//========================================================================
//--------------------------TransfoX--------------------------------------
//========================================================================

double TransfoX( double X, vector<double> &MinX ,vector<double> &MaxX ,vector<double> &ScalingBin ,vector<double> &NewMinX ,vector<double> &NewMaxX , vector<double> &Scale, int BinPerMM){
    /*
       This take as input the original bin of the histogram and the new bin defined by the function CalculateNewBin
       And output the new calculated X 
       */
    double XNew = -1000;
    for (int i= 0 ; i<=BinPerMM ; i++){
        if (X>= MinX[i] && X< MaxX[i]){
            double Xold = X;
            double Scale2 = (X-MinX[i])/(MaxX[i]-MinX[i]);
            XNew = Scale2 * (NewMaxX[i] - NewMinX[i])+NewMinX[i] ;
            if (XNew < NewMinX[i] || XNew > NewMaxX[i]  ){
                cout << " FUNCTION  " << endl;
                cout << i << endl ;
                cout << "MinX " << MinX[i] << " MaxX " << MaxX[i] << endl;
                cout << "Xold " << Xold  <<endl ;
                cout << "NewMinX " << NewMinX[i]<< " NewMaxX " << NewMaxX[i] << endl;
                cout << "Xnew  " <<  XNew  << endl;
                cout << endl << endl;
            } 
        }
    }
    return XNew;
} //end transfo x


void CalculateNewBin(vector<double> &MinX,vector<double> &MaxX,vector<double> &NewMinX, vector<double> &NewMaxX ,vector<double> &NewBinWidth , vector<double> &ScalingBin, vector<double> &Scale , vector<double> &NBin, int BinPerMM, int mm){
    //Tentative proportionelle au NBIN
    double SumBin = 0 ,NTotBin ,TotWidth=0. , NewTotWidth=0., IndexLastBin = BinPerMM;
    vector<double> BinWidth, NewBinWidthTheoretical;

    for (int i= 0 ; i<IndexLastBin ; i++){
        double d = i ; 
        double mmdouble = mm;
        MinX.push_back( mmdouble + d/(IndexLastBin));
        MaxX.push_back( mmdouble + (d+1)/(IndexLastBin));
        SumBin += NBin[i];
        BinWidth.push_back( MaxX[i] - MinX[i]);
        TotWidth += BinWidth[i];
    }   


    NTotBin = SumBin / (IndexLastBin);
    NewMinX[0] = MinX[0];

    for (int i= 0 ; i<IndexLastBin ; i++){
        ScalingBin.push_back(NBin[i] / NTotBin);
        NewBinWidthTheoretical.push_back(ScalingBin[i]*BinWidth[i]);

        if (i!=0){
            NewMinX[i] = NewMaxX[i-1];
        }
        double MaxXTemp = NewMinX[i] + BinWidth[i]; 
        Scale.push_back((NewMinX[i] + BinWidth[i] * ScalingBin[i])/MaxXTemp);
        NewMaxX[i] =(NewMinX[i] + BinWidth[i])* Scale[i];
        NewBinWidth.push_back(NewMaxX[i]-NewMinX[i]);
        NewTotWidth += NewBinWidth[i];

        if(false){
            cout << " itération " << i << endl;
            cout << " MinX " << MinX[i] << " MaxX " << MaxX[i] << endl;
            cout << " NewMin "  << NewMinX[i] << "  NewMax " << NewMaxX[i] << endl;
            cout << " Scaling  "<< Scale[i] <<  " NewBinWidth " << NewBinWidth[i] << endl;
            cout << " Width of bin " << NewBinWidth[i]<< "  expected width  " << NewBinWidthTheoretical[i] << endl;
            cout << " SumBin " << SumBin << endl;
            cout << endl << endl;
        }
    }
    NewMinX[IndexLastBin+1] = NewMaxX[IndexLastBin];
}

void GetBin(vector<double>&NBin, vector<TH1F*>&OldPosX, int StartBin, int BinPerMM ,int DetNbr )
{
    for (int bin = StartBin ; bin < StartBin + BinPerMM  ; bin++) {
        NBin.push_back(OldPosX[DetNbr]->GetBinContent(bin));
    }
}
