#include <TFile.h>
#include <TTree.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <TPaveStats.h>
#include <TStyle.h>
#include <TSystem.h>

int BeforeAfterLinearisationHist() {
    // Open the ROOT files
    TFile *cutfile = TFile::Open("cutBe10.root");
    TFile *file1 = TFile::Open("../../../root/analysis/Exc240Linea2228.root");

    // Retrieve the trees from the respective files
    TTree *tree1 = (TTree*)file1->Get("PhysicsTree");

    //Cut Be10 load
    TCut *Be10 = (TCut*)cutfile->Get("Be10");

    // Create histograms to hold the data from the branches
    TH1D *hist1 = new TH1D("Linearised", "Linearised", 500, -10, 30);
    TH1D *hist2 = new TH1D("Original", "Original", 500, -10, 30);
    TH1D *Theta = new TH1D("Theta", "Theta", 500, -0.4, 0.4);

    TH2D *XY1 = new TH2D("XY1", "XY1", 200 , -40 ,40, 200, -40 ,40);
    TH2D *XY2 = new TH2D("XY2", "XY2", 200 , -40 ,40, 200, -40 ,40);
    TH2D *XYLin1 = new TH2D("XYLin1", "XYLin1", 200 , -40 ,40, 200, -40 ,40);
    TH2D *XYLin2 = new TH2D("XYLin2", "XYLin2", 200 , -40 ,40, 200, -40 ,40);

    // Create a canvas
    TCanvas *c4 = new TCanvas("c4", "XYc", 1200, 600);
    TCanvas *c5 = new TCanvas("c5", "Thetac", 800, 600);
    TCanvas *c1 = new TCanvas("c1", "ComparisonLineaUnlinea", 1200, 600);
    TCanvas *c2 = new TCanvas("c2", "Canvas Title", 800, 600);
    TCanvas *c3 = new TCanvas("c3", "Canvas Title", 800, 600);


    // Set different line styles and colors for readability
    hist1->SetLineColor(kRed);
    hist1->SetLineWidth(2);
    hist1->SetLineStyle(1); // Solid line

    hist2->SetLineColor(kBlue);
    hist2->SetLineWidth(2);
    hist2->SetLineStyle(2); // Dashed line

    // Fill the histograms from the branches
    tree1->Draw("Ex240PuLin>>Linearised","Be10 && m_2alpha==0 && VAMOS_TS_hour>0","");
    tree1->Draw("Ex240Pu>>Original","Be10 && m_2alpha==0 && VAMOS_TS_hour >0 ","SAME");

    // Draw ===================================================
		c2->cd();
		hist2->Draw();
		
    c3->cd();
    hist1->Draw();

    c1->cd();
    tree1->Draw("Ex240PuLin>>Linearised","Be10 && m_2alpha==0&& VAMOS_TS_hour >0 && Telescope == 4","");
    tree1->Draw("Ex240Pu>>Original","Be10 && m_2alpha==0&& VAMOS_TS_hour >0 && Telescope == 4","SAME");


    // Draw custom text in place of histogram title
    TLatex latex;
    latex.SetNDC();
    latex.SetTextSize(0.04);
    latex.DrawLatex(0.44, 0.93, "Linearised Vs Original")  ;
    
    // Update the canvas to ensure the statistics boxes are created
    c1->Update();
    gSystem->ProcessEvents(); // Force the GUI event loop to update
    gStyle->SetOptTitle(0);
    // Draw statistics box for the first histogram
    TPaveStats *st1 = (TPaveStats*)hist1->GetListOfFunctions()->FindObject("stats");
    if (st1) {
        st1->SetTextColor(kRed);
        st1->SetX1NDC(0.70); // Move the statistics box for hist1
        st1->SetX2NDC(0.90);
        st1->SetY1NDC(0.75);
        st1->SetY2NDC(0.9);
    }

    // Draw statistics box for the second histogram
    TPaveStats *st2 = (TPaveStats*)hist2->GetListOfFunctions()->FindObject("stats");
    if (st2) {
        st2->SetTextColor(kBlue);
        st2->SetX1NDC(0.70); // Move the statistics box for hist2
        st2->SetX2NDC(0.90);
        st2->SetY1NDC(0.6);
        st2->SetY2NDC(0.75);
    }

    // Add a legend to distinguish between the histograms
    TLegend *legend = new TLegend(0.7, 0.5, 0.9, 0.6);
    legend->AddEntry(hist1, "Linearised", "l");
    legend->AddEntry(hist2, "Original", "l");
    legend->Draw();

    // Update the canvas to display the histograms and statistics boxes
    c1->Update();

    c4->cd();
    c4->Divide(2,2);
    c4->cd(1);
    tree1->Draw("PositionX:PositionY>>XY1"," m_2alpha==0 && VAMOS_TS_hour>0 && DetectorNumber == 1","colz");
    c4->cd(2);

    tree1->Draw("LinPositionX:LinPositionY>>XYLin1"," m_2alpha==0 && VAMOS_TS_hour>0 && DetectorNumber == 1","colz");

    c4->cd(3);

    tree1->Draw("PositionX:PositionY>>XY2"," m_2alpha==0 && VAMOS_TS_hour>0 && DetectorNumber == 2","colz");
    c4->cd(4);

    tree1->Draw("LinPositionX:LinPositionY>>XYLin2","m_2alpha==0 && VAMOS_TS_hour>0 && DetectorNumber == 2","colz");

    c5->cd();
    tree1->Draw("ThetaLab-ThetaLabLin>>Theta", "m_2alpha ==0 && VAMOS_TS_hour && Be10","" );
    gStyle->SetOptTitle(0);
    // Close the ROOT files
    //file1->Close();
    //file2->Close();

    return 0;
}

