#include <TCanvas.h>
#include <TChain.h>
#include "ProfileEvaluator.h"
#include "TICPhysics.h"
#include "TTimeData.h"
#include <TStyle.h>
#include <chrono>
#include <TH3.h>
#include <TH2.h>


void EvaluateProfileFromFile(const char* filename = "input.root", const char* profname = "prof", Double_t x = 0, Double_t y = 0) ;

vector<double> TxtToVector(const char *Path);


void CorrectXY(){

    ProfileEvaluator evaluator;
    evaluator.LoadProfile("Output/RatioProfile.root","ICOneZeroProfile");

    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");

    TTimeData *Time = new TTimeData();
    TICPhysics* IC = new TICPhysics() ;
    double FF_IC_X, FF_IC_Y, FF_V13, FF_DriftTime;



    vector<double> Toff13 , Toff14, Toff23, Toff24;
    const char* Path13 = "../../mwpc/Toff/output/Toff13.txt";
    const char* Path14 = "../../mwpc/Toff/output/Toff14.txt";
    const char* Path23 = "../../mwpc/Toff/output/Toff23.txt";
    const char* Path24 = "../../mwpc/Toff/output/Toff24.txt";

    Toff13 = TxtToVector(Path13);
    Toff14 = TxtToVector(Path14);
    Toff23 = TxtToVector(Path23);
    Toff24 = TxtToVector(Path24);


    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
    chain->SetBranchStatus("FF_IC_Y", true);
    chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);

    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC);

    chain->SetBranchStatus("Time", true);
    chain->SetBranchAddress("Time", &Time);

    TProfile2D *Pxyz = new TProfile2D("ICOneZeroProfile","ICOneZeroProfile",100,-1000.0,1000.0,100,-1200.0,1000.0);
    TProfile2D *PxyzCorr = new TProfile2D("PxyzCorr","PxyzCorr",100,-1000.0,1000.0,100,-1200.0,1000.0);


    //int Nentries = chain->GetEntries();
    int Nentries = 1000000;
    auto start = std::chrono::high_resolution_clock::now();

    for (int e = 0; e < Nentries; e++) {
        if (e % 100000  == 0 && e > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }

        chain->GetEntry(e);

        if(Time->GetMWPC13Mult()==1 && IC->fIC_raw[8] >0){
            UShort_t FPMW_Section = Time->GetSection_MWPC3(0);
            FF_DriftTime = 10* (IC->fIC_TS.at(0) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))) ;


            Double_t ICRatio =  IC->fIC_raw[1]/IC->fIC_raw[0];

            if(IC->fIC_raw[8] >0 && ICRatio <= 8 && ICRatio >0.1 ) {

                Double_t ICRatioCorr = ICRatio * 1.3 / evaluator.Evaluate(FF_IC_X,FF_DriftTime,true);
                if (ICRatioCorr >100 ) ICRatioCorr = 1.2;
                Pxyz->Fill(FF_IC_X,FF_DriftTime,ICRatio);
                PxyzCorr->Fill(FF_IC_X,FF_DriftTime,ICRatioCorr);
            }
        }
    
    }

    gStyle->SetCanvasPreferGL(true);
    TCanvas *c1 = new TCanvas("c1","c1");
    gStyle->SetPalette(kRainBow);
    Pxyz->SetXTitle("X");
    Pxyz->SetYTitle("DriftTime");
    Pxyz->SetZTitle("RatioIC");
	Pxyz->SetMinimum(1.0);
	Pxyz->SetMaximum(1.5);
	Pxyz->Draw("GL SURF2");    // Configure the 3D view
    c1->Update();
   
    TCanvas *c2 = new TCanvas("c2","c2");
    gStyle->SetPalette(kRainBow);
    PxyzCorr->SetXTitle("X");
    PxyzCorr->SetYTitle("DriftTime");
    PxyzCorr->SetZTitle("RatioIC");
	PxyzCorr->SetMinimum(1.0);
	PxyzCorr->SetMaximum(1.5);
	PxyzCorr->Draw("GL SURF2");    // Configure the 3D view

    TFile *ofile = new TFile("Output/RatioProfile.root" ,"recreate" );
    Pxyz ->Write();

}



void EvaluateProfileFromFile(const char* filename ,
        const char* profname ,
        Double_t x ,
        Double_t y ) {
    ProfileEvaluator evaluator;
    if (!evaluator.LoadProfile(filename, profname)) {
        printf("Failed to load profile\n");
        return;
    }

    evaluator.PrintInfo();
    Double_t z = evaluator.Evaluate(x, y, true);
    printf("z(%.2f, %.2f) = %.3f\n", x, y, z);

    // Quick visualization
    TCanvas* c = new TCanvas("c", "Profile View", 800, 400);
    c->Divide(2,1);

    c->cd(1);
    evaluator.GetProfile()->Draw("colz");

    c->cd(2);
    evaluator.GetProfile()->Draw("surf1");
}

vector<double> TxtToVector(const char *Path){
	string line;
	vector<double> values;
	ifstream file(Path); 

	if (file.is_open()) {
		while (std::getline(file, line)) {
			try {
				values.push_back(std::stod(line));
			} catch (const std::invalid_argument& e) {
				std::cerr << "Invalid number in line: " << line << '\n';
			}
		}
		file.close();
	} else {
		std::cerr << "Error opening file.\n";
	}

	return values;
}
