#include "TTimeData.h"
#include <TICPhysics.h>
#include <TCanvas.h>
#include <TChain.h>
#include <TF1.h>
#include <TFile.h>
#include <TH2.h>
#include <TSpline.h>
#include <TCutG.h>
#include <TStyle.h>
#include <fstream>
#include <vector>
#include "ProfileEvaluator.h"

using namespace std;
vector<double> TxtToVector(const char *Path);
int GetNumberKey(TFile *infile ,const char* ClassName);

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin , double
        XMax, double YMin, double YMax);
//////////////////////////////////////////////////////////////////////////////////////////
void DECorr(bool cut = false, bool spline = false) {
    //===========================================================================================================
    //                           Loading var
    //===========================================================================================================
    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");

    TICPhysics* IC = new TICPhysics() ;
    TTimeData *Time = new TTimeData();
    double FF_IC_X, FF_IC_Y, FF_V13, Y;

    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);

    chain->SetBranchStatus("FF_IC_Y", true);
    chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);

    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC); 
   
    chain->SetBranchStatus("FF_V13", true);
    chain->SetBranchAddress("FF_V13", &FF_V13); 
   
    chain->SetBranchStatus("Time", true);
    chain->SetBranchAddress("Time", &Time);

    vector<double> Toff13 , Toff14, Toff23, Toff24;
    const char* Path13 = "../../mwpc/Toff/output/Toff13.txt";
    const char* Path14 = "../../mwpc/Toff/output/Toff14.txt";
    const char* Path23 = "../../mwpc/Toff/output/Toff23.txt";
    const char* Path24 = "../../mwpc/Toff/output/Toff24.txt";

    Toff13 = TxtToVector(Path13);
    Toff14 = TxtToVector(Path14);
    Toff23 = TxtToVector(Path23);
    Toff24 = TxtToVector(Path24);

    ProfileEvaluator Profile;
    Profile.LoadProfile("Output/RatioProfile.root","ICOneZeroProfile");

    TCutG *CutZ;
    TCutG *CutZbis;
    TFile *fCut;
    if (cut) {
        fCut =  TFile::Open("Output/CutDeCorr.root");
        CutZbis = (TCutG*)fCut->Get("CutZbis") ;
        CutZ = (TCutG*)fCut->Get("CutZ") ;
    }

    TFile *fSpline; 
    TSpline3 *splineDEcorr, *splineDEbis;
    fSpline= new TFile("Output/splineDE.root","open");
    splineDEcorr = (TSpline3*) fSpline->Get("SplineDe_0");
    splineDEbis = (TSpline3*) fSpline->Get("SplineDebis");

    vector<TSpline3*> vsplineDE;
    // Get number of spline
    int SplineCountDe = 0 ;
    TIter next1(fSpline->GetListOfKeys());
    TKey* key1;

    while ((key1=(TKey*)next1())){
        if (std::string(key1->GetClassName()) == "TSpline3"){
            SplineCountDe ++;
        }
    }
    
    //Fill spline vector
    for (int i =0 ; i<SplineCountDe ; i++){
        vsplineDE.push_back((TSpline3*) fSpline->Get(Form("SplineDe_%d",i)));
    }

    //empty backup one
    for (int i =0 ; i<SplineCountDe ; i++){
        if (vsplineDE[i]==nullptr) vsplineDE.erase(vsplineDE.begin()+i);
    }


    //Defining output histo
    TH2F* hDE_E = new TH2F("DE_E","DE_E",1000,0,22000,1000,0,28000);
    TH2F* hDE_E_splined = new TH2F("DE_E_splined","DE_E_splined",1000,0,22000,1000,0,28000);
    TH2F* hDE_Ebis = new TH2F("DE_Ebis","DE_Ebis",1000,0,22000,1000,0,24000);


    TH2F* hDE_Y = new TH2F("DE_Y","DE_Y",1000,-1000,1000,1000,0,22000);
    TH2F* hDE_Y_splined = new TH2F("DE_Ysplined","DE_Ysplined",1000,-1000,1000,1000,0,22000);
    TH2F* hDE_Y_splined_nocut = new TH2F("DE_Ysplined_nocut","DE_Ysplined_nocut",1000,-1000,1000,1000,0,22000);
    TH2F* hIC_Y = new TH2F("IC_Y","IC_Y",1000,-1000,1000,1000,0,22000);
    TH2F* hIC_Y_splined = new TH2F("IC_Ysplined","IC_Ysplined",1000,-1000,1000,1000,0,22000);


    TH2F* hDE_Y_corr = new TH2F("DE_Ycorr","DE_Ycorr",1000,-1000,1000,1000,0,22000);
    TH2F* hDE_Y_corr_nocut = new TH2F("DE_Ycorr_nocut","DE_Ycorr_nocut",1000,-1000,1000,1000,0,22000);

    TH2F* hDE_Y_bis = new TH2F("DE_Ybis","DE_Ybis",1000,-1000,1000,1000,0,22000);
    TH2F* hDEbis_Y_corr = new TH2F("DE_Ybiscorr","DE_Ybiscorr",1000,-1000,1000,1000,0,22000);

    TH2F* hDE_E_corr = new TH2F("DE_Ecorr","DE_Ecorr",1000,0,22000,1000,0,28000);
    TH2F* hDE_Ebis_corr = new TH2F("DE_Ebiscorr","DE_Ebiscorr",1000,0,22000,1000,0,24000);

    TH2F* hDE_V = new TH2F("DE_V","DE_V",1000,0,5,1000,0,28000);

    // Def Lim spline
    double Ymin = -1000 , Ymax =1000;
    //===========================================================================================================
    //                                        Event Loop
    //===========================================================================================================
    int Nentries = chain->GetEntries();
    //int Nentries = 1000000;
    auto start = std::chrono::high_resolution_clock::now();

    for (int e = 0; e < Nentries; e++) {
        if (e % 100000  == 0 && e > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }

        chain->GetEntry(e);


        if (Time->GetMWPC13Mult() ==1 && IC->fIC_TS.size()>=8){ //only mult 1 event
            UShort_t FPMW_Section = Time->GetSection_MWPC3(0);
            double FF_DriftTime = 10* (IC->fIC_TS.at(0) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))) ;

            Double_t ICRatio =  IC->fIC_raw[1]/IC->fIC_raw[0];
            Double_t ICRatioCorr =  ICRatio *1.3 / Profile.Evaluate(FF_IC_X,FF_DriftTime,true);
            Double_t ICcorr_Y = IC->fIC_raw[0] / 1.3 * Profile.Evaluate(FF_IC_X,FF_DriftTime,true);


            double DE = 0 , DE_splined=0 ,DE_corr=0, DE_Ybis=0;
            double E =0;

            for (int seg = 0 ;  seg < sizeof(IC->fIC_raw)/sizeof(IC->fIC_raw[0]) ; seg++ ){
                if (seg >4){
                    E += double(IC->fIC_raw[seg]) ;
                }
            }



            // ********************* DE setter *********************
            double IC1corr = (IC->fIC_raw[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
            double DE_Bis = 0.5*(IC1corr+IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

            DE = 0.5*(double(IC->fIC_raw[0])+ double(IC->fIC_raw[1]) +double(IC->fIC_raw[2])+ double(IC->fIC_raw[3])) + double(IC->fIC_raw[4]);
            DE_splined = 0.5*(ICcorr_Y+ IC->fIC_raw[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

            //DE_splined = 0.5*( ICcorr_Y[1] +IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];


            //************* Cut ******************
            bool CutV = abs( FF_V13-3.94)<0.1;
            bool CutVbis =  abs( FF_V13-3.94)<0.1;

            //bool CutV = true;
            //bool CutVbis =  true;

            //***********Fill histo*************

            if (FF_DriftTime >Ymin && FF_DriftTime <Ymax ){

                //*************Init DE_E***************** 
                hDE_E->Fill(E,DE);
                hDE_E_splined->Fill(E,DE_splined);
                hDE_Ebis->Fill(E,DE_Bis);

                hDE_V->Fill(FF_V13,DE_splined);    

                //******* Coor DE ************
                if (spline== true  ){     
                    DE_corr = DE_splined * splineDEcorr->Eval(0) / splineDEcorr->Eval(FF_DriftTime);
                    hDE_E_corr->Fill(E,DE_corr);

                    DE_Ybis = DE_Bis * splineDEbis->Eval(0) / splineDEbis->Eval(FF_DriftTime);
                    hDE_Ebis_corr->Fill(E,DE_Ybis); 

                }   

                else {
                    DE_corr = DE_splined;
                    hDE_E_corr->Fill(E,DE_corr);

                    DE_Ybis = DE_Bis ;
                    hDE_Ebis_corr->Fill(E,DE_Ybis);  
                }

                bool CutCharge ;


                if (cut == true) CutCharge =  CutZ->IsInside(FF_V13,DE_splined);
                //bool CutChargeBis =  CutZ->IsInside(E,DE_Y_Recur);
                //bool CutVbis =  CutZbis->IsInside(E,DE_Bis);
                // DE_Y in a cut


                if (cut == true && CutV) {
                    if ( CutV) {
                        hDE_Y_corr_nocut->Fill(FF_DriftTime,DE_corr);
                        hDE_Y_splined_nocut->Fill(FF_DriftTime,DE_splined);
                        hDE_Y->Fill(FF_DriftTime,DE);
                        hDE_Y_splined->Fill(FF_DriftTime,DE_splined); 
                        hDE_Y_corr->Fill(FF_DriftTime,DE_corr);

                        hIC_Y->Fill(FF_DriftTime,IC->fIC_raw[0]);
                        hIC_Y_splined->Fill(FF_DriftTime,ICcorr_Y);
                    }
                }//end cut
                else {      
                    hDE_Y->Fill(FF_DriftTime,DE);
                    hDE_Y_splined->Fill(FF_DriftTime,DE_splined); 
                    hDE_Y_corr->Fill(FF_DriftTime,DE_corr);

                    hIC_Y->Fill(FF_DriftTime,IC->fIC_raw[0]);
                    hIC_Y_splined->Fill(FF_DriftTime,ICcorr_Y);
                }

                if (cut == true && CutVbis) {
                    hDE_Y_bis->Fill(Y,DE_Bis);
                    hDEbis_Y_corr->Fill(Y,DE_Ybis);
                }
                else{;}
                //end cut
            }//end cond Y
        } // End mult 1
    } //endl loop event

    // making spline

    TSpline3 *splineDE, *osplineDEbis;
    if (spline == true){
        splineDE = MakeSpline(hDE_Y_splined,0,Ymin,Ymax,0,23000); 
        osplineDEbis = MakeSpline(hDE_Y_bis,1,Ymin,Ymax,0,23000); 
        TFile *fSpline = new TFile("Output/splineDE.root","recreate");
        splineDE->SetName("SplineDe_0");
        splineDE->Write();
        osplineDEbis->SetName("SplineDebis");
        osplineDEbis->Write(); 
        fSpline->Close();
    }
    //===========================================================================================================
    //                                          Drawing histo
    //===========================================================================================================

    gStyle->SetPalette(kRainBow);

    TCanvas* c = new TCanvas("c","c",1500,1000);
    c->Divide(2,2);
    c->cd(1);
    gPad->SetLogz();
    hDE_Y->ProfileX()->Draw();
    c->cd(2);
    gPad->SetLogz();
    hDE_Y_splined->ProfileX()->Draw();
    c->cd(3);
    gPad->SetLogz();
    hIC_Y->ProfileX()->Draw();
    c->cd(4);
    gPad->SetLogz();
    hIC_Y_splined->ProfileX()->Draw();

    TCanvas *c4 = new TCanvas("c4","c4",1500,1000);
    c4->Divide(2);
    c4->cd(1);
    hDE_Y_splined_nocut->Draw();
    c4->cd(2);
    hDE_Y_corr_nocut->Draw();

    TCanvas *c5 = new TCanvas("c5","c5",1500,1000);
    hDE_E->Draw("colz");



    TCanvas *c1 = new TCanvas("c1","c1",1500,1000);
    hDE_V->Draw();

    TCanvas* c2 = new TCanvas("c2","c2",1500,1000);
    c2->Divide(3 + 1*spline);

    c2->cd(1);
    hDE_E->SetTitle("Unmodified");
    gPad->Modified();
    gPad->Update(); 
    hDE_E->Draw("colz");
    hDE_E->GetXaxis()->SetTitle("E");
    hDE_E->GetYaxis()->SetTitle("DE");

    c2->cd(2);
    hDE_Ebis->SetTitle("Online");
    gPad->Modified();
    gPad->Update();
    hDE_Ebis->Draw("colz");
    hDE_Ebis->GetXaxis()->SetTitle("E");
    hDE_Ebis->GetYaxis()->SetTitle("DE");

    c2->cd(3);
    hDE_E_splined->SetTitle("IC Corrected in DT");
    gPad->Modified();
    gPad->Update();
    hDE_E_splined->Draw("colz");
    hDE_E_splined->GetXaxis()->SetTitle("E");
    hDE_E_splined->GetYaxis()->SetTitle("DE");

    if ( spline == true){
        c2->cd(4);
        hDE_E_corr->SetTitle("DE Corrected");
        gPad->Modified();
        gPad->Update();    
        hDE_E_corr->Draw("colz"); 
        hDE_E_corr->GetXaxis()->SetTitle("E");
        hDE_E_corr->GetYaxis()->SetTitle("DE");
    }
} // End spline chio XY


int GetNumberKey(TFile *infile ,const char* ClassName){
    // Get number of spline
    int KeyCount = 0 ;
    TIter next(infile->GetListOfKeys());
    TKey* keyFit;

    while ((keyFit=(TKey*)next())){
        if (std::string(keyFit->GetClassName()) == ClassName){
            KeyCount ++;
        }
    }

    return KeyCount;

}

TSpline3* MakeSpline(TH2F* hInput, Int_t NCallee, double XMin = 0 , double
        XMax = 1000 , double YMin=0 , double YMax = 100) {

    TSpline3* gspline;

    TH1F* hProfile;
    TH1F* pfx; // extended hProfile

    TCanvas* canfit = new TCanvas(Form("CanSpline_%02d",NCallee),
            Form("canfit_%02d",NCallee), 0, 0, 1000, 500);


    //===========================================================================================================
    //                                            Init profile
    //===========================================================================================================

    // Create the TProfile
    hProfile = (TH1F*)hInput->ProfileX();
    hProfile->SetLineWidth(2);
    hProfile->SetDirectory(0);
    canfit->cd();

    hProfile->GetYaxis()->SetRangeUser(YMin, YMax);
    hProfile->Draw();
    //===========================================================================================================
    //                                          First and last bin to get to 6k
    //                                          event get promoted
    //===========================================================================================================
    int FirstBin, LastBin;
    double Treshold = 0;
    for (int bin =1 ; bin<hProfile->GetNbinsX(); bin++) {
        FirstBin = bin;
        if (hProfile->GetBinLowEdge(bin)> XMin) break;
    }
    for (int bin = hProfile->GetNbinsX(); bin>1 ; bin--) {
        LastBin = bin;
        if (hProfile->GetBinLowEdge(bin)< XMax) break;
    }
    //===========================================================================================================
    //                                          Init Extended profile function
    //===========================================================================================================
    // Create the extended TProfile
    pfx = new TH1F(
            Form("pfx_%02d", Int_t(NCallee/2)), Form("pfx_%02d", Int_t(NCallee/2)), hProfile->GetNbinsX(), hProfile->GetBinLowEdge(1),
            hProfile->GetBinLowEdge(hProfile->GetNbinsX()) + hProfile->GetBinWidth(hProfile->GetNbinsX()));
    pfx->SetLineColor(8);
    pfx->SetDirectory(0);
    //===========================================================================================================
    //                                           Fill extended profile
    //===========================================================================================================

    // fill the extended TProfile
    float newval, lastval, lasterr;
    for (int bin = 1; bin <= FirstBin; bin++) {
        newval = 0;
        pfx->SetBinContent(bin, newval);
    }


    for (int bin = FirstBin; bin <= LastBin; bin++) {
        newval = hProfile->GetBinContent(bin);
        if (newval != 0) {
            pfx->SetBinContent(bin, newval);
            pfx->SetBinError(bin, hProfile->GetBinError(bin));
            lastval = newval;
            lasterr = hProfile->GetBinError(bin);
        }
        else {
            pfx->SetBinContent(bin, lastval);
            pfx->SetBinError(bin, lasterr);
        }
    }
    pfx->Draw("same");

    gspline = new TSpline3(pfx);
    gspline->SetName(Form("fspline_%d", NCallee + 1));

    return gspline;

} // end makespline

vector<double> TxtToVector(const char *Path){
    string line;
    vector<double> values;
    ifstream file(Path); 

    if (file.is_open()) {
        while (std::getline(file, line)) {
            try {
                values.push_back(std::stod(line));
            } catch (const std::invalid_argument& e) {
                std::cerr << "Invalid number in line: " << line << '\n';
            }
        }
        file.close();
    } else {
        std::cerr << "Error opening file.\n";
    }

    return values;
}
