#include "TICPhysics.h"
#include "TTimeData.h"
#include <TChain.h>
#include <TFile.h>
#include <TH2.h>
#include <TH3.h>
#include <TStyle.h>
#include <TProfile2D.h>
#include <chrono>
vector<double> TxtToVector(const char *Path);

void VerifCorrelation(){

    //===========================================================================================================
    //                           Loading var
    //===========================================================================================================
    TChain* chain = new TChain("PhysicsTree");
    chain->Add("../../../root/analysis/VamosCalib247.root");

    TTimeData *Time = new TTimeData();
    TICPhysics* IC = new TICPhysics() ;
    double FF_IC_X, FF_IC_Y, FF_V13, FF_DriftTime;

  
    vector<double> Toff13 , Toff14, Toff23, Toff24;
    const char* Path13 = "../../mwpc/Toff/output/Toff13.txt";
    const char* Path14 = "../../mwpc/Toff/output/Toff14.txt";
    const char* Path23 = "../../mwpc/Toff/output/Toff23.txt";
    const char* Path24 = "../../mwpc/Toff/output/Toff24.txt";

    Toff13 = TxtToVector(Path13);
    Toff14 = TxtToVector(Path14);
    Toff23 = TxtToVector(Path23);
    Toff24 = TxtToVector(Path24);


    chain->SetBranchStatus("FF_IC_X", true);
    chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
    chain->SetBranchStatus("FF_IC_Y", true);
    chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);

    chain->SetBranchStatus("IC", true);
    chain->SetBranchAddress("IC", &IC);

    chain->SetBranchStatus("Time", true);
    chain->SetBranchAddress("Time", &Time);

    
    TH3F *ICXY = new TH3F("ICXY","ICXY",100,-550,400,100,-1000,1000,100,0,2);
    TProfile2D *Pxyz = new TProfile2D("ICOneZeroProfile","ICOneZeroProfile",100,-550.0,400.0,100,-900.0,500.0);
 
    int Nentries = chain->GetEntries();
    //int Nentries = 1000000;
    auto start = std::chrono::high_resolution_clock::now();

    for (int e = 0; e < Nentries; e++) {
        if (e % 100000  == 0 && e > 0 ) {
            auto now = std::chrono::high_resolution_clock::now();
            std::chrono::duration<double> elapsed = now - start;
            double avgTimePerIteration = elapsed.count() / e;
            double timeLeft = avgTimePerIteration * (Nentries - e);

            std::cout << "********** Estimated time left: " << int(timeLeft) << " seconds **********" << "\r" << flush;
        }

        chain->GetEntry(e);


        if(Time->GetMWPC13Mult()==1 && IC->fIC_raw[8] >0){
            UShort_t FPMW_Section = Time->GetSection_MWPC3(0);
            FF_DriftTime = 10* (IC->fIC_TS.at(0) - Time->GetTS_MWPC13(0)) - ((Time->GetTime_MWPC13(0)+Toff13.at(FPMW_Section))) ;


            Double_t ICRatio =  IC->fIC_raw[1]/IC->fIC_raw[0];

            if(IC->fIC_raw[8] >0 && ICRatio <= 3 && ICRatio >0.1 ) {
                ICXY->Fill(FF_IC_X,FF_DriftTime,ICRatio);
                Pxyz->Fill(FF_IC_X,FF_DriftTime,ICRatio);
            }
        }
    
    }
    gStyle->SetCanvasPreferGL(true);
    TCanvas *c = new TCanvas("c","c");
    gStyle->SetPalette(kRainBow);
    ICXY->SetXTitle("X");
    ICXY->SetYTitle("DriftTime");
    ICXY->SetZTitle("RatioIC");
	ICXY->SetMinimum(0);
	ICXY->Draw("GL");    // Configure the 3D view

	// Update the canvas
	c->Update();

    TCanvas *c1 = new TCanvas("c1","c1");
    gStyle->SetPalette(kRainBow);
    Pxyz->SetXTitle("X");
    Pxyz->SetYTitle("DriftTime");
    Pxyz->SetZTitle("RatioIC");
	Pxyz->SetMinimum(1.0);
	Pxyz->SetMaximum(1.5);
	Pxyz->Draw("GL SURF2");    // Configure the 3D view


    TFile *ofile = new TFile("Output/RatioProfile.root","recreate");
    Pxyz->Write();
}

vector<double> TxtToVector(const char *Path){
	string line;
	vector<double> values;
	ifstream file(Path); 

	if (file.is_open()) {
		while (std::getline(file, line)) {
			try {
				values.push_back(std::stod(line));
			} catch (const std::invalid_argument& e) {
				std::cerr << "Invalid number in line: " << line << '\n';
			}
		}
		file.close();
	} else {
		std::cerr << "Error opening file.\n";
	}

	return values;
}
