#include "IC_Dampen_Recombination.cxx"


void HistoFillerDE_E();
/** 
 * *  Draw the DE_E plot and save the histo into a file
 * */
void CutZ(bool GenDE=true){


  if (GenDE==true) HistoFillerDE_E() ;


  TFile* infile = TFile::Open("Output/DE_E.root");


  //===========================================================================================================
  //                    Load Histo
  //===========================================================================================================
  // Get number of TH2F
  int HistoCount = 0 ;
  TIter next(infile->GetListOfKeys());
  TKey* key;
  
  while ((key=(TKey*)next())){
    if (std::string(key->GetClassName()) == "TH2F"){
      HistoCount ++;
    }
  }

  vector<TH2F*> hDE_E;
  //for each histo in the file
  for (int i=0 ; i< HistoCount ; i++){
    if (i==0) hDE_E.push_back((TH2F*)infile->Get("DE_E_online"));    
    if (i==1) hDE_E.push_back((TH2F*)infile->Get("DE_Etot_online"));    
  }

  //===========================================================================================================
  //                                        Draw histo
  //===========================================================================================================

   TCanvas *cDE = new TCanvas("cDE","cDE",1000,500);
   cDE->Divide(2);
   cDE->cd(1);
   hDE_E[0]->Draw();
   cDE->cd(2);
   hDE_E[1]->Draw();
}

void HistoFillerDE_E(){


  //===========================================================================================================
  //                           Loading var
  //===========================================================================================================
  TChain* chain = new TChain("PhysicsTree");
  chain->Add("../../../root/analysis/VamosCalib241.root");


  TICPhysics* IC = new TICPhysics() ;
  double FF_IC_X, FF_IC_Y;

  chain->SetBranchStatus("FF_IC_X", true);
  chain->SetBranchAddress("FF_IC_X", &FF_IC_X);
  chain->SetBranchStatus("FF_IC_Y", true);
  chain->SetBranchAddress("FF_IC_Y", &FF_IC_Y);
  chain->SetBranchStatus("IC", true);
  chain->SetBranchAddress("IC", &IC); 

  //===========================================================================================================
  //                          Defining histo
  //===========================================================================================================

  TH2F* hDE_E_nocorr = new
    TH2F("DE_E_online","DE_E_online",1000,0,20000,1000,0,25000);
 TH2F* hDE_Etot = new
    TH2F("DE_Etot_online","DE_Etot_online",1000,0,50000,1000,0,25000);
 //===========================================================================================================
 //                     FILL HISTO
 //===========================================================================================================

  int Nentries = chain->GetEntries();

  for (int e = 0; e < Nentries; e++) {
    // Printing status
    if (e % 100000 == 0){
      cout <<  "Please wait, we are  " << e * 100. / Nentries << "% done ..\r" << flush;
    }
    chain->GetEntry(e);

    double DE = 0 ,E = 0, Etot = 0 ;

    //===========================================================================================================
    //                                Fill De from seg 1 to 3
    //===========================================================================================================
    for (int seg = 0 ;  seg < sizeof(IC->fIC_raw)/sizeof(IC->fIC_raw[0]) ; seg++ ){
      if(seg > 1 && seg < 4) DE += IC->fIC_raw[seg] ;
      else if (seg >4) E+= IC->fIC_raw[seg] ;
    }



    //===========================================================================================================
    //                                      Filling DE with ICO and 4
    //===========================================================================================================

    DE += IC->fIC_raw[0] ;

    DE = DE *0.5 + IC->fIC_raw[4];

    double IC1corr = (IC->fIC_raw[1]*(1-0.000686068*FF_IC_Y))*(1-4.88238e-05*FF_IC_Y+7.40395e-06*FF_IC_Y*FF_IC_Y);
    double DE_Bis = 0.5*(IC1corr+IC->fIC_raw[2]+IC->fIC_raw[3])+IC->fIC_raw[4];

    Etot = E+DE_Bis;
    //===========================================================================================================
    //                            Fill histo
    //===========================================================================================================

    hDE_E_nocorr->Fill(E,DE_Bis);
    hDE_Etot->Fill(Etot,DE_Bis);
  }

  TFile* outfile = new TFile("Output/DE_E.root","recreate");
  hDE_E_nocorr->Write(); 
  hDE_Etot->Write(); 
  outfile->Close();

}
