TChain* chain;
int nentries;
TTree* otree;
double DeltaE;
double DeltaE_back;
double Eres;
double Mass;
double Xcalc;
double Ycalc;
double Zcalc;
double XTarget;
double YTarget;
int Telescope;
int m_2alpha;

TCutG* cut_12C;

/////////////////////////////////////////////////////////
void LoadCuts(){
  TFile* ifile = new TFile("cut/cut_12C_calibrated.root");
  cut_12C = (TCutG*)ifile->FindObjectAny("cut_12C");
}

/////////////////////////////////////////////////////////
void LoadRootFile(){
  chain = new TChain("PhysicsTree");
  chain->Add("../../../root/analysis/run_228.root");

  chain->SetBranchStatus("Mass","true");
  chain->SetBranchAddress("Mass",&Mass);

  chain->SetBranchStatus("DeltaE","true");
  chain->SetBranchAddress("DeltaE",&DeltaE);

  chain->SetBranchStatus("DeltaE_back","true");
  chain->SetBranchAddress("DeltaE_back",&DeltaE_back);

  chain->SetBranchStatus("Eres","true");
  chain->SetBranchAddress("Eres",&Eres);

  chain->SetBranchStatus("Xcalc","true");
  chain->SetBranchAddress("Xcalc",&Xcalc);
  chain->SetBranchStatus("Ycalc","true");
  chain->SetBranchAddress("Ycalc",&Ycalc);
  chain->SetBranchStatus("Zcalc","true");
  chain->SetBranchAddress("Zcalc",&Zcalc);
 
  chain->SetBranchStatus("XTarget","true");
  chain->SetBranchAddress("XTarget",&XTarget);
 
  chain->SetBranchStatus("YTarget","true");
  chain->SetBranchAddress("YTarget",&YTarget);
 
  chain->SetBranchStatus("Telescope","true");
  chain->SetBranchAddress("Telescope",&Telescope);
 
  chain->SetBranchStatus("m_2alpha","true");
  chain->SetBranchAddress("m_2alpha",&m_2alpha);
 
}

/////////////////////////////////////////////////////////
void InitOutputTree(){
  otree = new TTree("tree","Select Tree");

  otree->Branch("DeltaE",&DeltaE,"DeltaE/D");
  otree->Branch("DeltaE_back",&DeltaE_back,"DeltaE_back/D");
  otree->Branch("Eres",&Eres,"Eres/D");
  otree->Branch("XTarget",&XTarget,"XTarget/D");
  otree->Branch("YTarget",&YTarget,"YTarget/D");
  otree->Branch("Xcalc",&Xcalc,"Xcalc/D");
  otree->Branch("Ycalc",&Ycalc,"Ycalc/D");
  otree->Branch("Zcalc",&Zcalc,"Zcalc/D");
  otree->Branch("Telescope",&Telescope,"Telescope/I");
}

/////////////////////////////////////////////////////////
void FillCalibratedSelectTree(){
  LoadRootFile();
  LoadCuts();

  nentries = chain->GetEntries();
  cout << "Number of entries: " << nentries << endl;

  TFile* ofile = new TFile("SelectTree_calibrated.root","recreate");
  InitOutputTree();
 
  for(int i=0; i<nentries; i++){
    chain->GetEntry(i);

    if(i%100000==0){
      cout << "\033[34m\r Processing tree..." << (double)i/nentries*100 << "\% done" << flush;
    }

    if(Telescope>0 && Xcalc != -1000 && Mass == -1000){
      if(cut_12C->IsInside(Eres,DeltaE) && DeltaE_back>10){
        otree->Fill(); 
      }
    }
  }

  otree->Write();
  delete ofile;
}
