import math
import numpy as np

det_size = 49.50

y_shift = 11.5
# Function to read configuration from a file
def read_config(file_path):
    config = {}
    with open(file_path, 'r') as file:
        for line in file:
            if '=' in line:
                key, value = line.split('=')
                config[key.strip()] = float(value.strip())
                if(key.strip()=='angle'):
                    config[key.strip()]*=np.pi/180
    return config

def compute_edges_det0(theta,z0):
    z_Y1 = z0
    y_Y1 =  y_shift
    z_Y16 = z0 + np.sign(z0)*det_size*np.sin(theta)
    y_Y16 = y_shift + det_size*np.cos(theta)
    X1_Y1 = np.array([-24.75,y_Y1,z_Y1])
    X1_Y16 = np.array([-24.75,y_Y16,z_Y16])
    X16_Y16 = np.array([24.75,y_Y16,z_Y16])
    X16_Y1 = np.array([24.75,y_Y1,z_Y1])
    return X1_Y1,X1_Y16,X16_Y16,X16_Y1

def compute_edges_det1(theta,z1):
    z_Y16 = z1
    y_Y16 =  y_shift
    z_Y1 = z1 + np.sign(z1)*det_size*np.sin(theta)
    y_Y1 = y_shift + det_size*np.cos(theta)
    X1_Y1 = np.array([-24.75,y_Y1,z_Y1])
    X1_Y16 = np.array([-24.75,y_Y16,z_Y16])
    X16_Y16 = np.array([24.75,y_Y16,z_Y16])
    X16_Y1 = np.array([24.75,y_Y1,z_Y1])
    return [X1_Y1,X1_Y16,X16_Y16,X16_Y1]


# Paths to your input and output files
config_path = "GeometryConfig.txt"  # Update this to your actual path
output_file_path = "EdinburghDSSD.detector"  # Update this to your actual path

def write_to_file(output_file):
    config = read_config(config_path)
    z0 = config['z0']
    z1 = config['z1']
    theta = config['angle']

    DET0_X1_Y1, DET0_X1_Y16, DET0_X16_Y16, DET0_X16_Y1 = compute_edges_det0(theta, z0)
    DET1_X1_Y1, DET1_X1_Y16, DET1_X16_Y16, DET1_X16_Y1 = compute_edges_det1(theta, z1)
    with open(output_file, 'w') as file:
        file.write("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n")
        file.write("EdinburghDSSD\n")
        file.write(f" X1_Y1= {DET0_X1_Y1[0]:.2f} {DET0_X1_Y1[1]:.2f} {DET0_X1_Y1[2]:.2f} mm\n")
        file.write(f" X1_Y16= {DET0_X1_Y16[0]:.2f} {DET0_X1_Y16[1]:.2f} {DET0_X1_Y16[2]:.2f} mm\n")
        file.write(f" X16_Y16= {DET0_X16_Y16[0]:.2f} {DET0_X16_Y16[1]:.2f} {DET0_X16_Y16[2]:.2f} mm\n")
        file.write(f" X16_Y1= {DET0_X16_Y1[0]:.2f} {DET0_X16_Y1[1]:.2f} {DET0_X16_Y1[2]:.2f} mm\n")
        file.write(f"\nEdinburghDSSD\n")
        file.write(f" X1_Y1= {DET1_X1_Y1[0]:.2f} {DET1_X1_Y1[1]:.2f} {DET1_X1_Y1[2]:.2f} mm\n")
        file.write(f" X1_Y16= {DET1_X1_Y16[0]:.2f} {DET1_X1_Y16[1]:.2f} {DET1_X1_Y16[2]:.2f} mm\n")
        file.write(f" X16_Y16= {DET1_X16_Y16[0]:.2f} {DET1_X16_Y16[1]:.2f} {DET1_X16_Y16[2]:.2f} mm\n")
        file.write(f" X16_Y1= {DET1_X16_Y1[0]:.2f} {DET1_X16_Y1[1]:.2f} {DET1_X16_Y1[2]:.2f} mm\n")

write_to_file(output_file_path)
