#ifndef Analysis_h 
#define Analysis_h
/*****************************************************************************
 * Copyright (C) 2009-2014    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Adrien MATTA  contact address: a.matta@surrey.ac.uk      *
 *                                                                           *
 * Creation Date  : march 2025                                               *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 * Class describing the property of an Analysis object                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#include"NPVAnalysis.h"
#include"NPEnergyLoss.h"
#include"NPReaction.h"
#include"RootOutput.h"
#include"RootInput.h"
#include "TSharcPhysics.h"
#include "TPlasticPhysics.h"
#include "TInitialConditions.h"
#include <TRandom3.h>
#include <TVector3.h>
#include <TMath.h>
#include <TCutG.h>

#include<fstream>
#include<vector>
class Analysis: public NPL::VAnalysis{
  public:
    Analysis();
    ~Analysis();

  public: 
    void Init();
    void TreatEvent();
    void End();

  void InitOutputBranch();
  void InitInputBranch();
  void ReInitValue();
  static NPL::VAnalysis* Construct();
 
  private:
  double Ex;
  double ELab;
  double ThetaLab;
  double ThetaCM;
  NPL::Reaction* myReaction;
  TInitialConditions* myInit ;
  //	Energy loss table: the G4Table are generated by the simulation
  EnergyLoss LightCD2;
  EnergyLoss LightAl;
  EnergyLoss LightSi;
  EnergyLoss BeamCD2;
  TVector3 BeamDirection;
  TVector3 TargetPosition;


  double TargetThickness ;
  // Beam Energy
  double OriginalBeamEnergy ; // AMEV
  // intermediate variable
  TRandom3 Rand ;
  int DetectorNumber  ;
  int RunNumber;
  int RunNumberMinor;
  double ThetaNormalTarget;
  double Energy ;
  double ThetaDetector;
  double Si_E_Sharc ;  
  double X_Sharc ;
  double Y_Sharc ;
  double Z_Sharc ;
  
  double X_Trifoil;
  double Y_Trifoil;

  TSharcPhysics* Sharc;
  TPlasticPhysics* Trifoil;

  TFile* HistoFile;
  TH1F* ThetaCM_emmitted;
  TH1F* ThetaCM_detected;
  TH1F* ThetaLab_emmitted;
  TH1F* ThetaLab_detected;

  TH1F* Efficiency_CM;
  TH1F* Efficiency_Lab;

  TH1F* SolidAngle_CM;
  TH1F* SolidAngle_Lab;

  TH2F* ThetaCM_emmitted_2D;
  TH2F* ThetaCM_detected_2D;
  TH2F* ThetaLab_emmitted_2D;
  TH2F* ThetaLab_detected_2D;

  TH2F* Kine_2D;

  TH2F* Efficiency_CM_2D;
  TH2F* Efficiency_Lab_2D;

  TH2F* SolidAngle_CM_2D;
  TH2F* SolidAngle_Lab_2D;

};
#endif
