/*****************************************************************************
 * Copyright (C) 2009-2016    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: XAUTHORX  contact address: XMAILX                        *
 *                                                                           *
 * Creation Date  : XMONTHX XYEARX                                           *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This class describe  TOGAXSI_GAGG analysis project                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *****************************************************************************/

#include<iostream>
using namespace std;
#include"Analysis.h"
#include"NPAnalysisFactory.h"
#include"NPDetectorManager.h"
////////////////////////////////////////////////////////////////////////////////
Analysis::Analysis(){
}
////////////////////////////////////////////////////////////////////////////////
Analysis::~Analysis(){
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::Init(){
  IC = new TInitialConditions;
  DC = new TInteractionCoordinates;
  RC = new TReactionConditions;

  InitOutputBranch();
  InitInputBranch();

  TOGAXSI_GAGG= (TTOGAXSI_GAGGPhysicsPhysics*) m_DetectorManager->GetDetector("TOGAXSI_GAGG");

  m_QFS = new NPL::QFS();
  m_QFS->ReadConfiguration(NOptionManager::getInstance()->GetReactionFile());

  myBeam = new NPL::QFS();
  myBeam->ReadConfigurationFile(NPOptionManager::getInstance()->GetReactionFile());
  InitialBeamEnergy = myBeam->GetEnergy();

  TargetThickness = 150*mm;
  string TargetMaterial = "LH2";
  string BeamName = NPL::ChangeNameToG4Standard(myBeam->GetName());

  rand = new TRandom3();
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::TreatEvent(){
  ReInitValue();
  unsigned int size = TOGAXSI_GAGG->GetEventMultiplicity();
}

////////////////////////////////////////////////////////////////////////////////
void Analysis::End(){
}
////////////////////////////////////////////////////////////////////////////////
void Analysis::InitOutputBranch() {
//  RootOutput::getInstance()->GetTree()->Branch("Ex",&Ex, "Ex/D");
//  RootOutput::getInstance()->GetTree()->Branch("E1",&E1, "E1/D");
//  RootOutput::getInstance()->GetTree()->Branch("E2",&E2, "E2/D");
//  RootOutput::getInstance()->GetTree()->Branch("Theta12",&Theta12, "Theta12/D");
//  RootOutput::getInstance()->GetTree()->Branch("ThetaCM",&ThetaCM, "ThetaCM/D");
//  RootOutput::getInstance()->GetTree()->Branch("VertexX",&VertexX, "VertexX/D");
//  RootOutput::getInstance()->GetTree()->Branch("VertexY",&VertexY, "VertexY/D");
//  RootOutput::getInstance()->GetTree()->Branch("VertexZ",&VertexZ, "VertexZ/D");
//  RootOutput::getInstance()->GetTree()->Branch("deltaX",&deltaX, "deltaX/D");
//  RootOutput::getInstance()->GetTree()->Branch("deltaY",&deltaY, "deltaY/D");
//  RootOutput::getInstance()->GetTree()->Branch("deltaZ",&deltaZ, "deltaZ/D");
}
////////////////////////////////////////////////////////////////////////////////
void Analysis::InitInputBranch() {
  RootInput::getInstance()->GetChain()->SetBranchAddress("InteractionCoordinates",&DC);
  RootInput::getInstance()->GetChain()->SetBranchAddress("ReactionConditions",&RC);
}
////////////////////////////////////////////////////////////////////////////////
void Analysis::ReInitValue() {
  Ex = -1000;
  E1 = -1000;
  E2 = -1000;
  Theta12 = -1000;
  ThetaCM = -1000;
  VertexX = -1000;
  VertexY = -1000;
  VertexZ = -1000;
  deltaX = -1000;
  deltaY = -1000;
  deltaZ = -1000;
  Distance = -1000;
  sumTheta = -1000;
  deltaPhi = -1000;
  alpha = -1000;
  Theta1 = -1000;
  Phi1 = -1000;
  Theta2 = -1000;
  Phi2 = -1000;
  Theta1s = -1000;
  Phi1s = -1000;
  Theta2s = -1000;
  Phi2s = -1000;
  TA = -1000;
  TAcalc = -1000;
}

////////////////////////////////////////////////////////////////////////////////
//            Construct Method to be pass to the DetectorFactory              //
////////////////////////////////////////////////////////////////////////////////
NPL::VAnalysis* Analysis::Construct(){
  return (NPL::VAnalysis*) new Analysis();
}

////////////////////////////////////////////////////////////////////////////////
//            Registering the construct method to the factory                 //
////////////////////////////////////////////////////////////////////////////////
extern "C"{
class proxy{
  public:
    proxy(){
      NPL::AnalysisFactory::getInstance()->SetConstructor(Analysis::Construct);
    }
};

proxy p;
}

