/* Predefine functions */
void C2S();
void C2S_Diagnosis();
vector<vector<double>> GetExpDiffCross(double Energy);
TH1F* PullThetaLabHistSubPSp(int i, double minTheta, double gatesize, int angBinSize);
//TH1F* PullThetaLabHist(int i, double minTheta, double gatesize, int angBinSize);
//TH1F* PullPhaseSpaceHist(int i, double minTheta, double gatesize, int angBinSize);
void Scale(TGraph* g , TGraphErrors* ex);
TGraph* TWOFNR(double E, double J0, double J, double n, double l, double j);
double ToMininize(const double* parameter);
TGraph* FindNormalisation(TGraph* theory, TGraphErrors* experiment);
TGraph* FindNormalisation(TGraph* theory, TGraph* theory2, TGraphErrors* experiment);
TList* peakFitList = new TList();

/* Global variables */
vector<Double_t> anglecentres, anglewidth;
TGraph* currentThry;
TGraph* currentThry2;
TGraphErrors* staticExp;
int indexE;
double globalS, globalSerr;
double globalS2, globalSerr2;
int numAngleBins;
double widthAngleBins;
double firstAngle;
double CSangleL, CSangleH;
double CSvertL,  CSvertH;
bool GammaGate = false;
double globGammaGate;
bool doMix = false;
bool doDoublet = false;
bool comparingTWOFNRs = false;
double globalBinning;
double SnTWOFNR = 7.607;

/* Output volume toggle */
bool loud = 1;

/* Scale method toggle */
bool scaleTogether = 1;

/* Strings for image */
string orbitalname;
string orbital;

/* Strings for SolidAngle input file */
string statename;
string inputdate;


////////////////////////////////////////////////////////////////////////////////
void WriteToFile(TGraphErrors* gr, string filename){
  int npoints = gr->GetN();
  ofstream file;
  file.open (filename.c_str());
  
  cout << " ============================ " << endl;
  cout << "  Writing to " << filename << " ..." << endl;

  file << " i \t x \t y \t xe \t ye" << endl;

  for(int i = 0; i < npoints; i++){
    double x = gr->GetPointX(i);    
    double y = gr->GetPointY(i);    
    double xe = gr->GetErrorX(i);    
    double ye = gr->GetErrorY(i);    

    file << i  << "\t"
	 << x  << "\t"
	 << y  << "\t"
	 << xe << "\t"
	 << ye << "\t"
	 << endl;

  }

  cout << "  Complete! " << endl;
  cout << " ============================ " << endl;

}

////////////////////////////////////////////////////////////////////////////////

TGraphErrors* GetExpDCS_ForFig(string filename){
  vector<double> x, dx, y, dy;
  ifstream f(filename.c_str());
  string line;
  while (getline(f, line)){
    stringstream ss(line);
    //      i   x   y  dx  dy
    double tA, tB, tC, tD, tE;
    if (f >> tA >> tB >> tC >> tD >> tE){
      //cout << tA << " " << tB << " " << tC << " " << tD << " " << tE << endl;
      x.push_back(tB);
      y.push_back(tC);
      dx.push_back(0.0);
      dy.push_back(tE);
    }
  }
  f.close();

  TGraphErrors* g = new TGraphErrors(x.size(), &(x[0]), &(y[0]), &(dx[0]), &(dy[0]));
  return g;
} 

////////////////////////////////////////////////////////////////////////////////
TGraph* GetSimDCS_ForFig(string filename){
  vector<double> x, dx, y, dy;
  ifstream f(filename.c_str());
  string line;
  while (getline(f, line)){
    stringstream ss(line);
    //      i   x   y  dx  dy
    double tA, tB, tC, tD, tE;
    if (f >> tA >> tB >> tC >> tD >> tE){
      //cout << tA << " " << tB << " " << tC << " " << tD << " " << tE << endl;
      x.push_back(tB);
      y.push_back(tC);
      dx.push_back(tD);
      dy.push_back(tE);
    }
  }
  f.close();

  TGraph* g = new TGraph(x.size(), &(x[0]), &(y[0]));
  return g;
}

void CanvasPartition(TCanvas *C,const Int_t Nx = 2,const Int_t Ny = 2,
                     Float_t lMargin = 0.15, Float_t rMargin = 0.05,
                     Float_t bMargin = 0.15, Float_t tMargin = 0.05);
double XtoPad(double x);
double YtoPad(double x);
 
////////////////////////////////////////////////////////////////////////////////
void CanvasPartition(TCanvas *C,const Int_t Nx,const Int_t Ny, Float_t lMargin, Float_t rMargin, Float_t bMargin, Float_t tMargin){
   if (!C) return;
 
   // Setup Pad layout:
   Float_t vSpacing = 0.0;
   Float_t vStep  = (1.- bMargin - tMargin - (Ny-1) * vSpacing) / Ny;
 
   Float_t hSpacing = 0.0;
   Float_t hStep  = (1.- lMargin - rMargin - (Nx-1) * hSpacing) / Nx;
 
   Float_t vposd,vposu,vmard,vmaru,vfactor;
   Float_t hposl,hposr,hmarl,hmarr,hfactor;
 
   for (Int_t i=0;i<Nx;i++) {
 
      if (i==0) {
         hposl = 0.0;
         hposr = lMargin + hStep;
         hfactor = hposr-hposl;
         hmarl = lMargin / hfactor;
         hmarr = 0.0;
      } else if (i == Nx-1) {
         hposl = hposr + hSpacing;
         hposr = hposl + hStep + rMargin;
         hfactor = hposr-hposl;
         hmarl = 0.0;
         hmarr = rMargin / (hposr-hposl);
      } else {
         hposl = hposr + hSpacing;
         hposr = hposl + hStep;
         hfactor = hposr-hposl;
        hmarl = 0.0;
         hmarr = 0.0;
      }
 
      for (Int_t j=0;j<Ny;j++) {
 
         if (j==0) {
            vposd = 0.0;
            vposu = bMargin + vStep;
            vfactor = vposu-vposd;
            vmard = bMargin / vfactor;
            vmaru = 0.0;
         } else if (j == Ny-1) {
            vposd = vposu + vSpacing;
            vposu = vposd + vStep + tMargin;
            vfactor = vposu-vposd;
            vmard = 0.0;
            vmaru = tMargin / (vposu-vposd);
         } else {
            vposd = vposu + vSpacing;
            vposu = vposd + vStep;
            vfactor = vposu-vposd;
            vmard = 0.0;
            vmaru = 0.0;
         }
 
         C->cd(0);
 
         auto name = TString::Format("pad_%d_%d",i,j);
         auto pad = (TPad*) C->FindObject(name.Data());
         if (pad) delete pad;
         pad = new TPad(name.Data(),"",hposl,vposd,hposr,vposu);
         pad->SetLeftMargin(hmarl);
         pad->SetRightMargin(hmarr);
         pad->SetBottomMargin(vmard);
         pad->SetTopMargin(vmaru);
 
         pad->SetFrameBorderMode(0);
         pad->SetBorderMode(0);
         pad->SetBorderSize(0);
 
         pad->Draw();
      }
   }
}

////////////////////////////////////////////////////////////////////////////////
void WriteToFile(TGraph* gr, string filename){
  int npoints = gr->GetN();
  ofstream file;
  file.open (filename.c_str());
  
  cout << " ============================ " << endl;
  cout << "  Writing to " << filename << " ..." << endl;

  file << " i \t x \t y \t xe \t ye" << endl;

  for(int i = 0; i < npoints; i++){
    double x = gr->GetPointX(i);    
    double y = gr->GetPointY(i);    
    double xe = gr->GetErrorX(i);    
    double ye = gr->GetErrorY(i);    

    file << i  << "\t"
	 << x  << "\t"
	 << y  << "\t"
	 << xe << "\t"
	 << ye << "\t"
	 << endl;

  }

  cout << "  Complete! " << endl;
  cout << " ============================ " << endl;

}

////////////////////////////////////////////////////////////////////////////////
void canclone(TCanvas* major, int padNum, string name){
  string minorName = "./CS2_Figures/" + name + ".root";
  TFile* minorFile = new TFile(minorName.c_str(),"READ");
  TCanvas* minor = (TCanvas*) minorFile->FindObjectAny("c_peakFits");
 
  major->cd(padNum);
  TPad *pad = (TPad*)gPad;
  minor->cd();
  TObject *obj, *clone;
  pad->Range(minor->GetX1(),minor->GetY1(),minor->GetX2(),minor->GetY2());
  pad->SetTickx(minor->GetTickx());
  pad->SetTicky(minor->GetTicky());
  pad->SetGridx(minor->GetGridx());
  pad->SetGridy(minor->GetGridy());
  pad->SetLogx(minor->GetLogx());
  pad->SetLogy(minor->GetLogy());
  pad->SetLogz(minor->GetLogz());
  pad->SetBorderSize(minor->GetBorderSize());
  pad->SetBorderMode(minor->GetBorderMode());
  minor->TAttLine::Copy((TAttLine&)*pad);
  minor->TAttFill::Copy((TAttFill&)*pad);
  minor->TAttPad::Copy((TAttPad&)*pad);

  TIter next(minor->GetListOfPrimitives());
  gROOT->SetSelectedPad(pad);
  while ((obj=next())) {
     clone = obj->Clone();
     obj->Draw("SAME");
     pad->GetListOfPrimitives()->Add(clone,obj->GetDrawOption());
  }
  pad->Modified();
  pad->Update();
  major->cd(padNum);
  pad->Draw();
}

////////////////////////////////////////////////////////////////////////////////
double GetNodes(double spdf){
  double nodes;
       if(spdf==0){ nodes=1; }
  else if(spdf==1){ nodes=0; }
  else if(spdf==2){ nodes=0; }
  else if(spdf==3){ nodes=0; }
  else{
    cout << " INPUT NODES::" << endl;
    cin >> nodes;
  }
  return nodes;
}

////////////////////////////////////////////////////////////////////////////////
void GenerateResidual(TGraphErrors* gdSdO, TGraph* Final){

  TGraphErrors* gResid = new TGraphErrors(*gdSdO);
  for(int n=0; n < gResid->GetN(); n++){
    double x = gdSdO->GetPointX(n);
    double residual = gdSdO->GetPointY(n) - Final->Eval(x);
    gResid->SetPoint(n,x,residual);
    gResid->SetPointError(n,0,gdSdO->GetErrorY(n));
  }
  TLine* markzero = new TLine(CSangleL,0.,CSangleH,0.);
  gResid->SetTitle("");
  gResid->GetXaxis()->SetRangeUser(CSangleL,CSangleH);
  gResid->GetXaxis()->SetNdivisions(512,kTRUE);
  gResid->GetYaxis()->SetTitle("Residuals");
  gResid->GetYaxis()->SetTitleSize(0.15);
  gResid->GetYaxis()->SetTitleOffset(0.36);
  gResid->GetYaxis()->SetLabelSize(0.08);
  gResid->GetYaxis()->SetNdivisions(305);
  gResid->GetXaxis()->SetTitleSize(0.15);
  gResid->GetXaxis()->SetTitleOffset(0.8);
  gResid->GetXaxis()->SetLabelSize(0.1);
  gResid->GetXaxis()->SetTickLength(0.1);
  gResid->Draw();
  markzero->SetLineStyle(2);
  markzero->Draw("same");

}

////////////////////////////////////////////////////////////////////////////////
TGraph* AddTGraphs(TGraph* a, TGraph* b){
  if(a->GetN() != b->GetN()){
    cout << "CANNOT ADD GRAPHS, UNEQUAL NUMBER OF POINTS" << endl;
    cout << a->GetN() << " != " << b->GetN() << endl;
    return 0;
  }

  TGraph* sum = (TGraph*) a->Clone();

  int maxN = a->GetN();
  for(int i = 0 ; i < maxN ; i++){
    double xa = a->GetPointX(i);
    double xb = b->GetPointX(i);
    if(xa!=xb){cout << "FAIL! Unequal X!" << endl; return 0;}

    double ya = a->GetPointY(i);
    double yb = b->GetPointY(i);
    sum->SetPoint(i,xa,ya+yb);
  }

  return sum;

}

////////////////////////////////////////////////////////////////////////////////
// Allow detailed choices (i.e. initial nucleus spin J0, and number/width/intial angle bins)
void C2S(double Energy, double Spin, double spdf, double angmom, double binning, string option,double J0, double nAngB, double wAngB, double fAngB ){
  /* Clean global variables, in case of multiple runs */
  indexE = 100;
  anglecentres.clear();
  anglewidth.clear();
  globalS=0.;
  globalSerr=0.;
  globalS2=0.;
  globalSerr2=0.;
  peakFitList->Clear();
  globalBinning = binning;

  //gROOT->SetBatch(1);

  double spdf2, angmom2 = 0;
  doMix = false;
  doDoublet = false;

  if(option=="mixed"){
    doMix = true;
    cout << BOLDBLUE 
	 << " || MIXING OPTION SELECTED  || \n"
	 << " || GIVE DETAILS OF 2ND OPT || \n"
	 << " || L = ...                 || \n"
	 << endl;
    cin >> spdf2;
    cout << " || J  = ...                || \n"
	 << endl;
    cin >> angmom2;
    cout << RESET;
  } else if (option=="doublet") {
    doDoublet = true;
  }

  if(binning < 0.005){
    cout << BOLDRED
	 << " BINNING IS BELOW 0.005 MeV! Exiting..."
	 << RESET
	 << endl;
    return;
  }

  /* Assign local variables */
  // s1/2 -> spdf = 0, angmom = 0.5
  // J0 is incident spin, which is 19O g.s. therefore J0 = 5/2
  //double J0 = 2.5; // NOW INPUT VARIABLE
  double ElasticNorm = 1.0, ElasticNormErr = 0.0;

  string backupFileName = "SolidAngleHistFiles/";
  string saFileName = "SolidAngleHistFiles/SAHF_Sim_19Odp_";

  // Define angle variables
  numAngleBins=nAngB; widthAngleBins=wAngB; firstAngle=fAngB; // NOW INPUT VARIABLE
  CSangleL = 100.;
  CSangleH = 160.;
  CSvertL  = 0.04; //0.01;
  CSvertH  = 100.; 

  //// Reduce angular range for high energy states
  //if(Energy>7.2){
  //  double tmax = (Energy-18.)/(-0.075);
  //  numAngleBins = (int)floor((double)(floor(tmax)-firstAngle)/widthAngleBins);
  //  cout << "!! TRIMMED NUMBER OF ANGLE BINS" << numAngleBins << endl;
  //  cout << "!! THEREFORE ANGE RANGE = " 
  //       << firstAngle << " to " 
  //       << firstAngle+(widthAngleBins*numAngleBins) 
  //       << endl;
  //}

  // Extract spin orbit name
  orbitalname.clear();
  orbital.clear();
  
  switch((int)spdf){
    case 0:
      orbitalname.append("s_{");
      orbital.append("s");
      break;
    case 1:
      orbitalname.append("p_{");
      orbital.append("p");
      break;
    case 2:
      orbitalname.append("d_{");
      orbital.append("d");
      break;
    case 3:
      orbitalname.append("f_{");
      orbital.append("f");
      break;
    default:
      cout << RED
	   <<"ERROR: SPDF SELECTION OUT OF RANGE" 
	   << endl;
  }
  orbitalname.append(to_string((int)(2*angmom)));
  orbitalname.append("/2}");
  orbital.append(to_string((int)(2*angmom)));
  orbital.append("2");

  // Number of nodes
  double nodes = GetNodes(spdf);
  double nodes2;
  if(doMix){
    nodes2 = GetNodes(spdf2);
  }

  /* Retrieve array index of the entered peak energy */
  /* numpeaks and Energy[] defined globally in KnownPeakFitter.h */
  bool found = 0;
  for(int i=0;i<numPeaks;i++){
  cout << i << "  " <<  Energy << "   " << means.at(i) << endl;
    if(abs(Energy-means.at(i))<0.01){
      cout << "========================================================" << endl;
      cout << "Identified as state #" << i << ", E = " << means.at(i) << endl;
      indexE = i;
      found = 1;
      stringstream ss;
      ss << setfill('0') << setw(4) << (int)((means.at(i)+0.0001)*1000.);
      statename = ss.str();
    }
  }
//  for(int i=0;i<numVoigt;i++){
//    if(abs(Energy-meansVoigt.at(i))<0.01){
//      cout << "========================================================" << endl;
//      cout << "Identified as state #" << i << ", E = " << meansVoigt.at(i) << endl;
//      indexE = numPeaks - 1 + i;
//      found = 1;
//      stringstream ss;
//      ss << setfill('0') << setw(4) << (int)((meansVoigt.at(i)+0.0001)*1000.);
//      statename = ss.str();
//    }
//  }
  if(!found){
    cout << "========================================================" << endl;
    cout << "NO STATE AT THAT ENERGY INDENTIFIED!! CHECK KNOWN PEAKS!!" << endl;
    return;
  }
  saFileName.append(statename);
  saFileName.append(".root");

  /* Solid Angle (from simulation) */
  TFile* file;
  ifstream infile(saFileName);
  ifstream backup(backupFileName);
  if(infile.good()){
    cout << GREEN << "Opening file " << saFileName << RESET << endl;
      file = TFile::Open(saFileName.c_str());
  } else if (backup.good()){
    cout << BOLDRED << "FAILED TO OPEN " << saFileName << endl;
    cout << "Open standard backup file " << backupFileName << endl;
    cout << RESET;
      file = TFile::Open(backupFileName.c_str());
  } else {
    cout << BOLDRED << "FAILED TO OPEN MAIN OR BACKUP SOLID ANGLE FILE" << endl;
    cout << RED << "Check SolidAngle file exists..." << RESET << endl;
  }

  string objName = "SolidAngle_Lab_MG";
  TH1F* SolidAngle = (TH1F*) file->FindObjectAny(objName.c_str());
  TCanvas* c_SolidAngle = new TCanvas("c_SolidAngle","c_SolidAngle",500,500);
  SolidAngle->Draw("HIST");
  SolidAngle->GetXaxis()->SetRangeUser(CSangleL,CSangleH);
  /* (canvas deleted after Area/SA calculation) */
 
  /* Area of experimental peaks */
  TCanvas* c_PeakArea = new TCanvas("c_PeakArea","c_PeakArea",500,500);
  vector<vector<double>> areaArray = GetExpDiffCross(means.at(indexE));
//  delete c_PeakArea;

  // Array: peakenergy, peakarea, areaerror, anglemin, anglemax
  if(loud){
    for(int i=0; i<areaArray.size();i++){
      cout << i << " " 
    	   << areaArray[i][0] << " " 
	   << areaArray[i][1] << " "
	   << areaArray[i][2] << " "
	   << areaArray[i][3] << " "
	   << areaArray[i][4] << endl;
    }
  }

  /* AoSA = Area/Solid Angle [#/msr] */
  /* dSdO = Experimental Diff. Cross Sect. (Area/Solid Angle *Norm) [mb/msr] */
  vector<Double_t> AoSA, AoSAerr;
  vector<Double_t> expDCS, expDCSerr;
  for(int i=0; i<areaArray.size();i++){
    int binmin = SolidAngle->FindBin(areaArray[i][3]+0.0001);
    int binmax = SolidAngle->FindBin(areaArray[i][4]-0.0001);

    anglecentres.push_back(((areaArray[i][4]-areaArray[i][3])/2.)+areaArray[i][3]);
    anglewidth.push_back(areaArray[i][4]-areaArray[i][3]);

    double tempsum=0, tempsumerr=0;
    for(int x=binmin;x<binmax+1;x++){
      tempsum += SolidAngle->GetBinContent(x);
      tempsumerr += SolidAngle->GetBinError(x);
      if(loud){cout << x << endl;}
    }
    if(loud){cout << "TEST CHECK " << tempsum << " +- " << tempsumerr << endl;}

    double SAerr;
    double SA = SolidAngle->IntegralAndError(binmin,binmax,SAerr);
    SA = SA*1000.;       //sr->msr
    SAerr = SAerr*1000.; //sr->msr

    /* Area over Solid angle ONLY */
    AoSA.push_back(areaArray[i][1]/SA);
    AoSAerr.push_back((areaArray[i][1]/SA) 
		    * sqrt( pow(areaArray[i][2]/areaArray[i][1],2) 
			    + pow(SAerr/SA,2)));

    /* Differential Cross Section */
    /* NOTE: DON'T INCLUDE NORM ERROR IN ERR PROPAGATION AS IT IS SYSTEMATIC! */
    double tempvalue = (areaArray[i][1]/SA)*ElasticNorm; 
    double temperror = tempvalue
		     * sqrt( pow(areaArray[i][2]/areaArray[i][1],2)
		           + pow(SAerr/SA,2)
			   );
    //if(temperror<tempvalue){ // exclude very large error bars
    expDCS.push_back(tempvalue);
    expDCSerr.push_back(temperror);
    //}

    if(loud){
      cout << "Angle " << areaArray[i][3] << " to " << areaArray[i][4] 
	   << ", centre " << anglecentres[i]
	   << ": Area = " << areaArray[i][1] << " +- " << areaArray[i][2] << " cnts" 
	   << "  SA = " << SA << " +- " << SAerr << " msr" 
           << "  Area/SA = " << AoSA[i] << " +- " << AoSAerr[i] << " counts/msr"
	   << setprecision(5)
           << "  Norm = " << ElasticNorm << " +- " << ElasticNormErr
	   << " (don't include norm err in propagation)"
           << "  dSdO = " << tempvalue << " +- " << temperror  
	   << setprecision(3)
	   << endl;
    }

  }
  //delete c_SolidAngle;
  
  gROOT->SetBatch(0);

  /* Graph of Area/Solid Angle*/
  TCanvas* c_AoSA = new TCanvas("c_AoSA","c_AoSA",500,500);
  c_AoSA->SetLogy();
  TGraphErrors* gAoSA = new TGraphErrors(
		  anglecentres.size(), //n
		  &(anglecentres[0]), &(AoSA[0]), //x, y
		  0, &(AoSAerr[0]) );  //errX, errY 
  gAoSA->SetTitle("Area/SolidAngle");
  gAoSA->GetXaxis()->SetTitle("ThetaLab [deg]");
  gAoSA->GetYaxis()->SetTitle("Counts/#Omega [counts/msr]");
  gAoSA->Draw();

  /* Graph of experimental diff. cross sect (dSigma/dOmega) */
  TCanvas* c_dSdO = new TCanvas("c_dSdO","c_dSdO",500,500);
  c_dSdO->SetLogy();
  TGraphErrors* gdSdO = new TGraphErrors(
		  anglecentres.size(),
		  &(anglecentres[0]), &(expDCS[0]),
		  0, &(expDCSerr[0]) );
  gdSdO->SetTitle("Differential Cross Section");
  gdSdO->GetXaxis()->SetTitle("#theta_{lab} [deg]");
  gdSdO->GetXaxis()->SetTitleOffset(1.2);
  gdSdO->GetXaxis()->SetTitleSize(0.04);
  gdSdO->GetYaxis()->SetTitle("d#sigma/d#Omega [mb/msr]");
  gdSdO->GetYaxis()->SetTitleOffset(1.2);
  gdSdO->GetYaxis()->SetTitleSize(0.04);
  gdSdO->Draw();
  c_dSdO->Update();

  /* TWOFNR diff. cross section, in mb/msr */ 
  double meanTWOFNR = means.at(indexE);
  if(meanTWOFNR > SnTWOFNR){meanTWOFNR = SnTWOFNR-0.001;}
  TCanvas* c_TWOFNR = new TCanvas("c_TWOFNR","c_TWOFNR",500,500);
  c_TWOFNR->SetLogy();
  
  TGraph* TheoryDiffCross;  TGraph* TheoryDiffCross2; 
  TGraph* TDCS_s;  TGraph* TDCS_p;  TGraph* TDCS_d;  TGraph* TDCS_f;  TGraph* TDCS_g;  TGraph* TDCS_h;
  if(comparingTWOFNRs==0){ //normal state
    TheoryDiffCross = TWOFNR(meanTWOFNR, Spin, nodes, spdf, angmom, J0); 
    TheoryDiffCross->GetYaxis()->SetTitle("d#sigma/d#Omega [mb/msr]"); //msr set in func above
    TheoryDiffCross->GetXaxis()->SetTitle("ThetaLab [deg]");
    TheoryDiffCross->Draw();

    if(doMix){
      TheoryDiffCross2 = TWOFNR(meanTWOFNR, Spin, nodes2, spdf2, angmom2, J0); 
      TheoryDiffCross2->SetLineColor(kBlue);
      TheoryDiffCross2->Draw("same");
    }

    /** TEMP **/
    if(loud){
      cout << "UNSCALED THEORY DIFF CROSS SECTION EVALUATED AT DATA POINTS:::" << endl;
      cout << setprecision(6);
      for(int i=0; i<numAngleBins; i++){
        cout << anglecentres.at(i) << "\t" << TheoryDiffCross->Eval(anglecentres.at(i)) << endl;
      }
      cout << setprecision(3);
      cout << "......................" << endl;
    }
  } else { // running for ALL ell-transfers, to compare shape
    cout << " TWOFNR s-wave..." << endl;  TDCS_s = TWOFNR(meanTWOFNR, 2, 1, 0, 0.5, 2.5);  TDCS_s->SetTitle("TDCS_s");
    cout << " TWOFNR p-wave..." << endl;  TDCS_p = TWOFNR(meanTWOFNR, 0, 0, 1, 0.5, 0.5);  TDCS_p->SetTitle("TDCS_p");
    cout << " TWOFNR d-wave..." << endl;  TDCS_d = TWOFNR(meanTWOFNR, 0, 0, 2, 2.5, 2.5);  TDCS_d->SetTitle("TDCS_d");
    cout << " TWOFNR f-wave..." << endl;  TDCS_f = TWOFNR(meanTWOFNR, 1, 0, 3, 3.5, 2.5);  TDCS_f->SetTitle("TDCS_f");
    cout << " TWOFNR g-wave..." << endl;  TDCS_g = TWOFNR(meanTWOFNR, 2, 0, 4, 4.5, 2.5);  TDCS_g->SetTitle("TDCS_g");
    cout << " TWOFNR h-wave..." << endl;  TDCS_h = TWOFNR(meanTWOFNR, 3, 0, 5, 5.5, 2.5);  TDCS_h->SetTitle("TDCS_h");
  }
  /** **** **/

  /* Using Chi2 minimizaiton */
  if(loud){cout << "USING CHI2 MINIMIZAITON..." << endl;}
  TCanvas* c_Chi2Min = new TCanvas("c_Chi2Min","c_Chi2Min",500,500);
  gStyle->SetPadLeftMargin(0.12);
  gStyle->SetPadRightMargin(0.03);

  TPad *pad1 = new TPad("pad1","pad1",0,0.25,1,1);
  TPad *pad2 = new TPad("pad2","pad2",0,0,1,0.25);
  pad1->SetTopMargin(0.1);
  pad1->SetBottomMargin(0.00001);
  pad1->SetBorderMode(0);
  pad1->SetLogy();
  pad1->SetTickx();
  pad1->SetTicky();
  pad2->SetTopMargin(0.00001);
  pad2->SetBottomMargin(0.3);
  pad2->SetBorderMode(0);
  pad2->SetTickx();
  pad2->SetTicky();
  pad1->Draw();
  pad2->Draw();
  pad1->cd();

  TGraph* Final;
  TGraph* Fs; TGraph* Fp; TGraph* Fd; TGraph* Ff; TGraph* Fg; TGraph* Fh;
  if(comparingTWOFNRs==0){ //normal state
    TGraphErrors* gdSdO2 = new TGraphErrors(*gdSdO);
    gdSdO2->SetName("gdSdO2");
    gdSdO2->SetTitle("gdSdO2");


    if(doMix){
      Final = FindNormalisation(TheoryDiffCross,TheoryDiffCross2,gdSdO);
    } else {
      Final = FindNormalisation(TheoryDiffCross,gdSdO);
    }

    Final->SetName("Final");
    Final->SetTitle("Final");

    gdSdO->SetLineColor(kRed);
    gdSdO->SetMarkerColor(kRed);
    gdSdO->SetMarkerStyle(21);
  } else { // running for ALL ell-transfers, to compare shape
    Fs = FindNormalisation(TDCS_s,gdSdO);
    Fp = FindNormalisation(TDCS_p,gdSdO);
    Fd = FindNormalisation(TDCS_d,gdSdO);
    Ff = FindNormalisation(TDCS_f,gdSdO);
    Fg = FindNormalisation(TDCS_g,gdSdO);
    Fh = FindNormalisation(TDCS_h,gdSdO);
    Fs->SetLineColor(kViolet);  
    Fp->SetLineColor(kRed);     
    Fd->SetLineColor(kGreen);   
    Ff->SetLineColor(kBlue);    Ff->SetLineStyle(7);
    Fg->SetLineColor(kCyan);    Fg->SetLineStyle(7);
    Fh->SetLineColor(kOrange);  Fh->SetLineStyle(7);

    gdSdO->SetLineColor(kBlack);
    gdSdO->SetMarkerColor(kBlack);
    gdSdO->SetMarkerStyle(21);
  }

  /* Construct file name string */
  /**/  ostringstream tempstream;
  /**/  if(means.at(indexE)<1.0){tempstream << 0;}
  /**/  tempstream << (int) (means.at(indexE)*1000);
  /**/  tempstream << "_" << orbital; 
  /**/  tempstream << "_spin" << Spin;
  /**/  string tempstr = tempstream.str();
  /* Construct hist title string */
  /**/  ostringstream textstream;
  /**/  textstream << std::fixed << setprecision(3);
  /**/  textstream << "   " << means.at(indexE);
  /**/  textstream << " MeV, ";
  /**/  textstream <<  orbitalname;
  /**/  textstream << ", spin " << (int)Spin;
  /**/	textstream << " --> scale (not norm.) = " << globalS 
  /**/	           << " +- " << globalSerr;
  /**/	if(doMix){
  /**/	  textstream << " & " << globalS2 
  /**/	             << " +- " << globalSerr2;
  /**/  }	
  /**/  string textstring = textstream.str(); 

  gdSdO->SetTitle(textstring.c_str());
  gdSdO->GetYaxis()->SetTitleOffset(1.3);
  gdSdO->GetYaxis()->SetTitleSize(0.042);
  gdSdO->GetXaxis()->SetRangeUser(CSangleL,CSangleH); 
  gdSdO->GetYaxis()->SetRangeUser(CSvertL,CSvertH); 
  gdSdO->GetXaxis()->SetNdivisions(512,kTRUE);
  gdSdO->Draw("AP");

  if(comparingTWOFNRs==0){ //normal state
    Final->Draw("SAME");
  } else {
    Fs->Draw("SAME");
    Fp->Draw("SAME");
    Fd->Draw("SAME");
    Ff->Draw("SAME");
    Fg->Draw("SAME");
    Fh->Draw("SAME");
  }

  pad2->cd();

  if(comparingTWOFNRs==0){ //normal state
    GenerateResidual(gdSdO,Final);
  }

  string savestring1 = "./C2S_Outputs/"+tempstr+".root";
  string savestring2 = "./C2S_Outputs/"+tempstr+".pdf";
  c_Chi2Min->SaveAs(savestring1.c_str());
  c_Chi2Min->SaveAs(savestring2.c_str());

  cout << YELLOW
       << " -----  C2S = (2J+1)S = (2*" << (int)Spin << "+1)S = (" 
       << (int)((2.*Spin)+1.) << ")S = " 
       << ((2.*Spin)+1.) * globalS << "  ----- " 
       << RESET << endl;


  //delete c_AoSA;
  //delete c_dSdO;
}

////////////////////////////////////////////////////////////////////////////////
// For simple states, don't require detailed choices
void C2S(double Energy, double Spin, double spdf, double angmom, double binning, string option){
  C2S(Energy, Spin, spdf, angmom, binning, option, 2.5, 19, 2.0, 106.0);
}


////////////////////////////////////////////////////////////////////////////////
// Comparing one state to many TWOFNRs
void C2S_CompareTWOFNRs(double Energy, double Spin, double spdf, double angmom, double binning, string option, double J0, double nAngB, double wAngB, double fAngB ){
  comparingTWOFNRs = true;
  C2S(Energy, Spin, spdf, angmom, binning, option, J0, nAngB, wAngB, fAngB);
  comparingTWOFNRs = false;
}

////////////////////////////////////////////////////////////////////////////////
vector<vector<double>> GetExpDiffCross(double Energy){
  cout << "========================================================" << endl;
  vector<vector<double>> AllPeaks_OneGate;
  vector<vector<double>> OnePeak_AllGates;
  /****CHANGE ANGLE GATING****/
  //double widthAngleBins = 2.5;
  //double firstAngle = 105.;
  /***************************/
  double x[numAngleBins], y[numAngleBins];
  //TList* list = new TList();

  double trackScale = 0.0;
  /* Determine scaling factor for PhaseSpace */
  TCanvas* c_ExSubPSpace = new TCanvas("c_ExSubPSpace","c_ExSubPSpace",500,500);
  //if(scaleTogether){
  //  cout << BLUE << endl;
  //  cout << " SUBTRACTING PHASE SPACE TOGETHER! " << endl;
  //  TH1F* baseEx = PullThetaLabHist(0,firstAngle,widthAngleBins);
  //  TH1F* basePS = PullPhaseSpaceHist(0,firstAngle,widthAngleBins);
  //  for(int i=1; i<numAngleBins;i++){
  //    TH1F* addEx = PullThetaLabHist(i,firstAngle,widthAngleBins); baseEx->Add(addEx,1.);
  //    TH1F* addPS = PullPhaseSpaceHist(i,firstAngle,widthAngleBins); basePS->Add(addPS,1.);
  //  }
  //
  ////  /* Subtract flat background equal to smallest bin in range */
  ////  baseEx->GetXaxis()->SetRange(baseEx->FindBin(-1.),baseEx->FindBin(1.));
  ////  double minValueInRange = baseEx->GetBinContent(baseEx->GetMinimumBin());
  ////  baseEx->GetXaxis()->UnZoom();
  ////  cout << "Subtracting background of " << minValueInRange << endl;
  ////  for(int b=1; b<baseEx->GetNbinsX() ; b++){
  ////    baseEx->SetBinContent(b,baseEx->GetBinContent(b)-minValueInRange);
  ////  }
  // 
  //  // Fix error where basePS = 600, baseEx=150
  //  if(loud){
  //    cout << "baseEx bins: " << baseEx->GetNbinsX() << endl;
  //    cout << "basePS bins: " << basePS->GetNbinsX() << endl;
  //  }
  //  if(basePS->GetNbinsX()!=baseEx->GetNbinsX()){
  //    cout << basePS->GetNbinsX() / baseEx->GetNbinsX() << endl;
  //    int ratio = basePS->GetNbinsX() / baseEx->GetNbinsX();
  //    basePS->Rebin(ratio);
  //  }
  //  if(loud){
  //    cout << "baseEx bins: " << baseEx->GetNbinsX() << endl;
  //    cout << "basePS bins: " << basePS->GetNbinsX() << endl;
  //  }

  //  /* Begin scaling within range, track changes */
  //  trackScale = 0.7;
  //  basePS->Scale(trackScale);
  //  int numAngleBinsScale = baseEx->GetNbinsX();
  //  int nbinlow = basePS->FindBin(7.5); int nbinhigh = basePS->FindBin(10.0);
  //  for(int b=nbinlow; b<nbinhigh; b++){
////cout << "----" << endl;
  //    if(baseEx->GetBinContent(b) > 0.0 && basePS->GetBinContent(b) > baseEx->GetBinContent(b)){
  //    while(basePS->GetBinContent(b) > baseEx->GetBinContent(b)){
  //        basePS->Scale(0.99999);
  //        trackScale *= 0.99999;
////cout << "Ex: " << baseEx->GetBinContent(b) << "   PS: " << basePS->GetBinContent(b) << endl;
  //      }
  //    }
  //  }

  //  TH1F* finalEx = (TH1F*) baseEx->Clone();
  //  finalEx->SetName("ExSubPSpace");
  //  finalEx->SetTitle("ExSubPSpace");
  //  finalEx->Add(basePS,-1.);
  //  baseEx->SetLineColor(kBlack);  baseEx->Draw("hist");
  //  basePS->SetLineColor(kOrange); basePS->Draw("hist same");
  //  finalEx->SetLineColor(kRed);   finalEx->Draw("hist same");
  //  if(loud){cout << "PhaseSpace -> ExpData scaling = " << trackScale << endl;}
  //  cout << RESET << endl;
  //}

  /* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
  //for(int i=0; i<1; i++){
  for(int i=0; i<numAngleBins;i++){
    double min = firstAngle + (i*widthAngleBins);
    double max = min + widthAngleBins;
    cout << GREEN << endl;
    cout << "===================================" << endl;
    cout << " RUNNING SUB AND FIT " << endl;
    cout << "min: " << min << " max: " << max << endl;
  
    stringstream tmp; tmp << fixed << setprecision(0); 
    tmp << "c_peakFits_" << min << "_" << max; 
    string tmp2 = tmp.str();
    TCanvas* c_peakFits = new TCanvas("c_peakFits",tmp2.c_str(),500,500);

    /* Retrieve theta-gated Ex TH1F from file GateThetaLabHistograms.root */
    /* To change angle gates, run GateThetaLab_MultiWrite() */
    TH1F *gate; // HERE< TAKE VERSION WIH PSP ALREADY REMOVED!
    gate = PullThetaLabHistSubPSp(i,firstAngle,widthAngleBins,(int) widthAngleBins);

    ///* Retrieve theta-gated Ex TH1F from file GateThetaLabHistograms.root */
    ///* To change angle gates, run GateThetaLab_MultiWrite() */
    //TH1F *gate, *pspace;
    //gate = PullThetaLabHist(i,firstAngle,widthAngleBins,(int) widthAngleBins);
    //pspace = PullPhaseSpaceHist(i,firstAngle,widthAngleBins,(int) widthAngleBins);

    ///* Scale the Phase Space at this angle... */
    ///* ... using DrawExThetaLab_SubPSp value */
    //gate->Add(pspace,-0.134256);
    //for(int b=0; b<gate->GetNbinsX();b++){
    //  if(gate->GetBinContent(b)<0){
    //    if(loud){cout << "flattening bin " << gate->GetBinCenter(b) << ", from " << gate->GetBinContent(b) << " to 0" << endl;}
    //    gate->SetBinContent(b,0);
    //  }
    //}
    //
    /* ... for all angles together */
    //if(scaleTogether){
    //  if(loud){ cout << "gate bins: " << gate->GetNbinsX() << endl;
    //            cout << "pspc bins: " << pspace->GetNbinsX() << endl; }
    //  // Fix error where basePS = 600, baseEx=150
    //  if(pspace->GetNbinsX()!=gate->GetNbinsX()){
    //    if(loud){cout << pspace->GetNbinsX() / gate->GetNbinsX() << endl;}
    //    int ratio = pspace->GetNbinsX() / gate->GetNbinsX();
    //    pspace->Rebin(ratio);
    //  }
    //  if(loud){ cout << "gate bins: " << gate->GetNbinsX() << endl;
    //            cout << "pspc bins: " << pspace->GetNbinsX() << endl; }
    //  cout << "SUMMATION: Subtracting phase space, scaled by " << trackScale << endl;
    //  gate->Add(pspace,-trackScale);
    //} 
    //
    ///* ... or seperately for each angular bin */
    ///* NOTE THAT THIS DOES NOT ACCOUNT FOR FLAT BACKGROUND */
    //else {
    //  if(pspace->Integral() > 50.){ // Non-garbage histogram
    //    trackScale=0.7;
    //    pspace->Scale(trackScale);
    //    int numAngleBins = gate->GetNbinsX();
    //    for(int b=0; b<numAngleBins; b++){
    //      if(loud){cout << " FROM " << pspace->GetBinContent(b) << 
    //    	              " > " << gate->GetBinContent(b); 
    //      }
    //      while(pspace->GetBinContent(b) > gate->GetBinContent(b)){
    //        pspace->Scale(0.9999);
    //        trackScale*=0.9999;
    //      }
    //      if(loud){cout << " TO " << pspace->GetBinContent(b) << 
    //      	             " > " << gate->GetBinContent(b) << endl;
    //      }
    //    }
    //    if(loud){cout << " !!! SCALE FOR THIS ANGLE = " << trackScale << endl;}
    //    cout << "INDIVIDUAL: Subtracting phase space, scaled by " << trackScale << endl;
    //    gate->Add(pspace,-1);
    //  }
    //}
    // Retrieve array containing all fits, for one angle gate. //
    // Specific peak of interest selected from the vector by   //
    // global variable indexE                                  //
    gate->Draw();
    AllPeaks_OneGate = FitKnownPeaks_RtrnArray(gate); 
    

    /* Write PS-subtracted spectrum to list */
    //list->Add(gate);
    //list->Add(c_peakFits);
    gate->GetXaxis()->SetRangeUser(-1.0,+10.0);
    string filename = "./C2S_Outputs/";
    filename += tmp2 + ".root";
    c_peakFits->SaveAs(filename.c_str());
    //auto tempfile = new TFile(filename.c_str(),"UPDATE"); //Reopen newly made file
    //auto templist = new TList();
    //templist->Add();


    /* Check correct OneGate vector is selected */
    if(loud){
      cout << "area of " << means.at(indexE) << " = "
           << AllPeaks_OneGate[indexE][1] 
	   << " +- " << AllPeaks_OneGate[indexE][2] 
	   << endl;
    }
cout << "here.... indexE = " << indexE << endl;
    /* Add min and max angle to end of relevant OneGate vector */
    AllPeaks_OneGate[indexE].push_back(min);
    AllPeaks_OneGate[indexE].push_back(max);

cout << "here...." << endl;
    /* Push relevant OneGate vector to end of AllGates vector */
    OnePeak_AllGates.push_back(AllPeaks_OneGate[indexE]);
    //delete c_peakFits; 
  }

cout << "here...." << endl;
  /* Write PS-subtracted spectrum to file */
  //TFile* outfile = new TFile("GateThetaLab_ExMinusGatePhaseSpace.root","RECREATE");
  //list->Write("GateThetaLab_ExMinusPhaseSpace",TObject::kSingleKey);

cout << "here...." << endl;
  return OnePeak_AllGates;
}

////////////////////////////////////////////////////////////////////////////////
TH1F* PullThetaLabHistSubPSp(int i, double minTheta, double gatesize, int angBinSize){
  string name = "./C2S_Outputs/GateThetaLabHist_Experiment_" + to_string(angBinSize) + "deg_SubPSp.root";
//string name = "./C2S_Outputs/GateThetaLabHist_Parallel_Experiment_SubPSp_" + to_string(angBinSize) + "deg.root";
  TFile* file = new TFile(name.c_str(),"READ");
  string histname = "TLab" 
	          + to_string((int)(minTheta+(i*gatesize))) + "to" 
		  + to_string((int)(minTheta+((i+1)*gatesize)));
  cout << "Loading " << name << " -> " << histname << endl;
  
  TList *list = (TList*)file->Get("GateThetaLabHistograms");
  TH1F* hist = (TH1F*)list->FindObject(histname.c_str());
  hist->GetXaxis()->SetRangeUser(-2.,10.);

  double pulledBin = hist->GetBinWidth(10);
  int ratio = (int)(globalBinning / pulledBin);
  hist->Rebin(ratio);
  file->Close();

  return hist;
}

//////////////////////////////////////////////////////////////////////////////////
//TH1F* PullThetaLabHist(int i, double minTheta, double gatesize, int angBinSize){
//  string name = "./C2S_Outputs/GateThetaLabHist_Experiment_" + to_string(angBinSize) + "deg.root";
//  //string name = "./C2S_Outputs/GateThetaLabHist_Experiment_" + to_string(angBinSize) + "deg_SubPSp.root";
//  TFile* file = new TFile(name.c_str(),"READ");
//  string histname = "TLab" 
//	          + to_string((int)(minTheta+(i*gatesize))) + "to" 
//		  + to_string((int)(minTheta+((i+1)*gatesize)));
//  cout << "Loading " << name << " -> " << histname << endl;
//  
//  TList *list = (TList*)file->Get("GateThetaLabHistograms");
//  TH1F* hist = (TH1F*)list->FindObject(histname.c_str());
//  hist->GetXaxis()->SetRangeUser(-2.,10.);
//
//  double pulledBin = hist->GetBinWidth(10);
//  int ratio = (int)(globalBinning / pulledBin);
//  hist->Rebin(ratio);
//  file->Close();
//
//  return hist;
//}
//
//////////////////////////////////////////////////////////////////////////////////
//TH1F* PullPhaseSpaceHist(int i, double minTheta, double gatesize, int angBinSize){
//  string name = "./C2S_Outputs/GateThetaLabHist_PhaseSpace_" + to_string(angBinSize) + "deg.root";
//  TFile* file = new TFile(name.c_str(),"READ");
//  string histname = "TLab" 
//	          + to_string((int) (minTheta+(i*gatesize))) + "to" 
//		  + to_string((int) (minTheta+((i+1)*gatesize)));
//  cout << "Loading " << name << " -> " << histname << endl;
//  
//  TList *list = (TList*)file->Get("GateThetaLabHistograms");
//  TH1F* hist = (TH1F*)list->FindObject(histname.c_str());
//  file->Close();
//
//  double pulledBin = hist->GetBinWidth(10);
//  int ratio = (int)(globalBinning / pulledBin);
//  hist->Rebin(ratio);
//
//  return hist;
//}

////////////////////////////////////////////////////////////////////////////////
void Scale(TGraph* g , TGraphErrors* ex){
  double scale;
  double mean = 0 ;
  double* eX = ex->GetX();
  double* eY = ex->GetY();
  double totalW = 0;
  double W = 0;
  for(int i = 0 ; i < ex->GetN() ; i++){
    if(eY[i]>1 && eY[i] <1e4){
      // Incremental Error weighted average
      W = 1./ex->GetErrorY(i);
      scale = eY[i]/g->Eval(eX[i]);
      totalW +=W;
      mean = mean + W*(scale - mean)/(totalW);
    }
  }

  //scaleTWOFNR = mean;
  if(loud){
    cout << "SCALED THEORY BY " << mean << endl;
    cout << " therefore S = " << 1/mean << " ??" << endl;
  }
  
  double* x = g->GetX();
  double* y = g->GetY();
  for(unsigned int i = 0 ; i < g->GetN() ; i++)
    g->SetPoint(i,x[i],y[i]*mean);
}

////////////////////////////////////////////////////////////////////////////////
double Chi2(TGraph* theory, TGraphErrors* exper){
  double Chi2 = 0;
  double chi = 0;

  //cout << setprecision(8);
  //for(int i = 1 ; i < exper->GetN() ; i++){
  for(int i = 0 ; i < exper->GetN() ; i++){
    if(exper->GetPointY(i)>1.0e-4){//10){ //0){
//      cout << setprecision(6) 
//	   << "COMPARE::::: " 
//	   << exper->GetPointY(i) << " +- " << exper->GetErrorY(1) 
//	   << " TO " 
//	   << theory->Eval(anglecentres[i]) 
//	   << endl;
    
      //////////////////////////////////////////////////////////////////////////////////////////
      // ---------------------------------------------------------------------------------------
      // | Louis Lyons pg 102:
      // |   Least squares should minimize sum(y_obs - y_th / err)^2
      // |   Err SHOULD in principle be error in THEORY
      // |   However it is common practive to use error in OBS for various reasons (pg 103-104)
      // ---------------------------------------------------------------------------------------
      //////////////////////////////////////////////////////////////////////////////////////////

      //Chi2 += pow((exper->GetPointY(i) - theory->Eval(anglecentres[i])), 2 ) / theory->Eval(anglecentres[i]);
      Chi2 += pow(((exper->GetPointY(i) - theory->Eval(anglecentres[i]))) / exper->GetErrorY(i) , 2 );
    }
  }
  if(loud){cout << setprecision(10) << "Chi2 = " << Chi2 << "  #dof = " << exper->GetN()-1 << "   Chi2/#dof = " << Chi2/(double)exper->GetN()-1 << setprecision(4)<< endl;}
  return Chi2;
}

////////////////////////////////////////////////////////////////////////////////
double ToMininize(const double* parameter){
  static int f = 0 ;
  TGraph* g = new TGraph();
  if(doMix){
    double* X = currentThry->GetX(); // gets valies from global g1 = tgraph passed to find norm
    double* Y1 = currentThry->GetY();
    double* Y2 = currentThry2->GetY();
    for(int i = 0 ; i < currentThry->GetN() ; i++){
      g->SetPoint(g->GetN(),X[i],parameter[0]*Y1[i] + parameter[1]*Y2[i]); // scales this tgraph by parameter
    }
  } else {
    double* X = currentThry->GetX(); // gets valies from global g1 = tgraph passed to find norm
    double* Y = currentThry->GetY();
    for(int i = 0 ; i < currentThry->GetN() ; i++){
      g->SetPoint(g->GetN(),X[i],parameter[0]*Y[i]); // scales this tgraph by parameter
    }
  }

  double chi2  = Chi2(g,staticExp);  //compares theory tgraph to experimental tgrapherrors by chi2
  return chi2;
}

////////////////////////////////////////////////////////////////////////////////
TGraph* FindNormalisation(TGraph* theory, TGraphErrors* experiment){
  /* (dSdO)meas = S * (dSdO)calc */

  // Set global variable
  currentThry = theory;
  staticExp = experiment;

  // Construct minimiser
  const char* minName ="Minuit";const char* algoName="Migrad";
  ROOT::Math::Minimizer* min =
    ROOT::Math::Factory::CreateMinimizer(minName, algoName);
  min->SetValidError(true);

  // Number of parameters (should only be 1 for me)
  int mysize = 1;

  // Create funciton wrapper for minimizer
  // a IMultiGenFunction type
  ROOT::Math::Functor f(&ToMininize,mysize);
  min->SetFunction(f);

  //int SpecFactorUpperLimit = 2;
  int SpecFactorUpperLimit = 20;

  // Set range of parameter(??)
  double* parameter = new double[mysize];
  for(unsigned int i = 0 ; i < mysize ; i++){
    parameter[i] = 0.5;
    char name[4];
    sprintf(name,"S%d",i+1);
    min->SetLimitedVariable(i,name,parameter[i],0.01,0,SpecFactorUpperLimit);
  }
 
  //min->SetPrintLevel(3);

  ///// TO IMPROVE: FIND WAY OF OBTAINING NDF AND PRINT CHI2/NDF /////

  // Minimise
  min->ProvidesError();
  min->SetValidError(true);
  min->Minimize();
  const double *xs = min->X();
  const double *err = min->Errors(); 

  cout << "IS THE MINIMIZATION GOOD? -> " << min->IsValidError() << endl;

  // Write out
  for(int i = 0  ; i < mysize ; i++){
    cout << Form("S%d=",i+1) << xs[i] << "(" << err[i] << ")" << endl;
  }

  /* Store S value in global variable, to access for drawing on plots */
  globalS = xs[0];
  globalSerr = err[0];

  // Return the Fitted CS
  TGraph* g = new TGraph(); 
  double* X = theory->GetX();
  double* Y = theory->GetY();
  if(loud){
    cout << setprecision(8);
    cout << "Start: X[0] = " << theory->GetPointX(4) << " Y[0] = " << theory->GetPointY(4) << endl;
    cout << "multip by " << xs[0] << endl;
  }
  
  for(int i=0; i<theory->GetN(); i++){ g->SetPoint(i,X[i],xs[0]*Y[i]); }

  if(loud){
    cout << "End:   X[0] = " << g->GetPointX(4) << " Y[0] = " << g->GetPointY(4) << endl;
    cout << setprecision(3);
  }

  return g;
}

////////////////////////////////////////////////////////////////////////////////
TGraph* FindNormalisation(TGraph* theory, TGraph* theory2, TGraphErrors* experiment){
  /* (dSdO)meas = S * (dSdO)calc */

  // Set global variable
  currentThry = theory;
  currentThry2 = theory2;
  staticExp = experiment;

  // Construct minimiser
  const char* minName ="Minuit";const char* algoName="Migrad";
  ROOT::Math::Minimizer* min =
    ROOT::Math::Factory::CreateMinimizer(minName, algoName);
  min->SetValidError(true);

  // Number of parameters (should only be 1 for me)
  int mysize = 1;
  if(doMix){mysize=2;}

  // Create funciton wrapper for minimizer
  // a IMultiGenFunction type
  ROOT::Math::Functor f(&ToMininize,mysize);
  min->SetFunction(f);
 
  // Set range of parameter(??)
  double* parameter = new double[mysize];
  for(unsigned int i = 0 ; i < mysize ; i++){
    //parameter[i] = 0.8;
    parameter[i] = 10.;
    char name[4];
    sprintf(name,"S%d",i+1);
    //min->SetLimitedVariable(i,name,parameter[i],0.01,0,10);
    min->SetLimitedVariable(i,name,parameter[i],0.01,0,20);
  }
 
  // Minimise
  min->Minimize();
  const double *xs = min->X();
  const double *err = min->Errors(); 

  // Write out
  for(int i = 0  ; i < mysize ; i++){
    cout << Form("S%d=",i+1) << xs[i] << "(" << err[i] << ")" << endl;
  }

  /* Store S value in global variable, to access for drawing on plots */
  globalS = xs[0];
  globalSerr = err[0];
  globalS2 = xs[1];
  globalSerr2 = err[1];

  // Return the Fitted CS
  TGraph* g = new TGraph(); 
  double* X = theory->GetX();
  double* Y1 = theory ->GetY();
  double* Y2 = theory2->GetY();
  if(loud){
    cout << setprecision(8);
    cout << "Start: X[0] = " << theory->GetPointX(4) << " Y[0] = " << theory->GetPointY(4) << endl;
    cout << "multip by " << xs[0] << endl;
    cout << "Start: X[1] = " << theory2->GetPointX(4) << " Y[0] = " << theory2->GetPointY(4) << endl;
    cout << "multip by " << xs[1] << endl;
  }
  
  for(int i=0; i<theory->GetN(); i++){
    g->SetPoint(i, X[i], xs[0]*Y1[i] + xs[1]*Y2[i]); 
  }

  if(loud){
    cout << "End:   X    = " << g->GetPointX(4) << " Y    = " << g->GetPointY(4) << endl;
    cout << setprecision(3);
  }

  return g;
}

////////////////////////////////////////////////////////////////////////////////
void C2S(double Energy, double Spin, double spdf, double angmom, double binning, double GammaGateEnergy){
  GammaGate=true;
  globGammaGate = GammaGateEnergy;

  C2S(Energy, Spin, spdf, angmom, binning, "");
}
