#!/bin/bash

cd ~/Programs/nptoolv3/Projects/e775s/analysis;
rm ./InputFiles/Auto.runtotreat
rm ./InputFiles/Auto.reaction

sp=" "
dfile="./InputFiles/mugast_2024-09-06.detector"
rfile="./InputFiles/Auto.reaction"
#rfile="./InputFiles/SpecificSim.reaction"

#Variables
E=$(echo "scale=3; $2/1000" | bc -l)
X="-3.8"
Y="-0.4"

#====================================================
# Symbolic link for mugast map
#----------------------------------------------------
# Remove old links
 rm ~/Programs/nptoolv3/NPLib/Detectors/Mugast/MugastMap.h
 rm ~/Programs/nptoolv3/NPLib/Detectors/Mugast/MugastReverseMap.h
#----------------------------------------------------
# Link "SIMU"
#echo "SYMBOLIC LINK: SIMU"
#ln -s ~/Programs/nptoolv3/NPLib/Detectors/Mugast/Zanon_Simu_MugastMap.h ~/Programs/nptoolv3/NPLib/Detectors/Mugast/MugastMap.h
#ln -s ~/Programs/nptoolv3/NPLib/Detectors/Mugast/Zanon_Simu_MugastReverseMap.h ~/Programs/nptoolv3/NPLib/Detectors/Mugast/MugastReverseMap.h
#----------------------------------------------------
# Link "ORIG"
 echo "SYMBOLIC LINK: ORIG"
 ln -s ~/Programs/nptoolv3/NPLib/Detectors/Mugast/Zanon_Orig_MugastMap.h ~/Programs/nptoolv3/NPLib/Detectors/Mugast/MugastMap.h
 ln -s ~/Programs/nptoolv3/NPLib/Detectors/Mugast/Zanon_Orig_MugastReverseMap.h ~/Programs/nptoolv3/NPLib/Detectors/Mugast/MugastReverseMap.h
#====================================================

#====================================================
#Boilerplate
echo -e "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" 		  	      >> ./InputFiles/Auto.reaction
echo -e "Beam" 						  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""Particle=""$sp""19O" 			  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""ExcitationEnergy=""$sp""0""$sp""MeV" 	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""Energy=""$sp""151.82""$sp""MeV" 		  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""SigmaEnergy=""$sp""0.0""$sp"" MeV" 	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""SigmaThetaX=""$sp""0.0""$sp""deg" 	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""SigmaPhiY=""$sp""0.0""$sp""deg"	 	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""SigmaX=""$sp""1.5""$sp""millimeter" 	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""SigmaY=""$sp""3.0""$sp""millimeter"	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""MeanThetaX=""$sp""0""$sp""deg"	  	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""MeanPhiY=""$sp""0""$sp""deg" 		  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""MeanX=""$sp""$X""$sp""millimeter" 	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""MeanY=""$sp""$Y""$sp""millimeter" 	  	      >> ./InputFiles/Auto.reaction
echo -e "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" 		  	      >> ./InputFiles/Auto.reaction
echo -e "TwoBodyReaction" 				  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""Beam=""$sp""19O" 				  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""Target=""$sp""2H" 			  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""Light=""$sp""1H" 			  	  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""Heavy=""$sp""20O" 			  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""ExcitationEnergyLight=""$sp""0.0""$sp""MeV" 	      >> ./InputFiles/Auto.reaction
echo -e "$sp""ExcitationEnergyHeavy=""$sp""$E""$sp""MeV"  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""LabCrossSectionPath=""$sp""flatbackwards.txt""$sp""CSR" >> ./InputFiles/Auto.reaction
echo -e "$sp""ShootLight=""$sp""1" 			  	      >> ./InputFiles/Auto.reaction
echo -e "$sp""ShootHeavy=""$sp""0" 			  	      >> ./InputFiles/Auto.reaction

#====================================================

cmake ./;
make -j6;

nameconstruct=$1"_"$2
echo "Name: $nameconstruct"
echo "Reaction file: $rfile"
echo "Detector file: $dfile"

for x in 0 #1 2 3 4 5 6 7 8 9
do
  outname=$nameconstruct"-"$x
 #npsimulation -D $dfile -E $rfile -B ./InputFiles/runSimulation.mac --random-seed $RANDOM -O $outname;
  npsimulation -D $dfile -E $rfile -B ./InputFiles/runSimSmall.mac --random-seed $RANDOM -O $outname;
done

outfile="Sim_"$nameconstruct

echo "TTreeName"      >> ./InputFiles/Auto.runtotreat
echo " SimulatedTree" >> ./InputFiles/Auto.runtotreat 
echo "RootFileName"   >> ./InputFiles/Auto.runtotreat 

directory='/home/paxman/Programs/nptoolv3/Outputs/Simulation/'
defineState='S'$2

echo "Defining state = "$defineState

for x in 0 #1 2 3 4 5 6 7 8 9 
do
  echo "$sp""$directory""$nameconstruct""-""$x"".root" >> ./InputFiles/Auto.runtotreat
done

npanalysis --definition Sim --definition $defineState -R ./InputFiles/Auto.runtotreat -E $rfile -D $dfile -O $outfile;

mvname="./SolidAngleHistFiles/SAHF_"$outfile".root"
mv ./SolidAngleHistFiles/SAHF_New.root $mvname

